package javawrapper;

import org.xml.sax.Attributes;

/**
 * This class organizes the seed predation behaviors.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 22, 2007: Added neighborhood seed predation (LEM)
 * <br>May 13, 2008: Added linked seed predation (LEM)
 */

public class SeedPredationBehaviors extends BehaviorTypeBase {
	/**Neighborhood seed predation - masting "p0"*/
	protected ModelVector mp_fNeighPredMastingP0 = new ModelVector(
			"Neighborhood Predation - Masting \"p0\"",	"pr_neighPredMastingP0", 
			"pr_npmp0Val", 0, ModelVector.FLOAT);

	/**Neighborhood seed predation - non-masting "p0"*/
	protected ModelVector mp_fNeighPredNonMastingP0 = new ModelVector(
			"Neighborhood Predation - Non-Masting \"p0\"",	
			"pr_neighPredNonMastingP0", "pr_npnmp0Val", 0, ModelVector.FLOAT);
	
	/**Neighborhood seed predation - min neighbor DBH*/
  protected ModelVector mp_fNeighPredMinDbh = new ModelVector(
      "Neighborhood Predation - Minimum Neighbor DBH (cm)", 
      "pr_neighPredMinNeighDBH", "pr_npmndVal", 0, ModelVector.FLOAT, true);
  
  /**Neighborhood seed predation - whether species is counted towards masting*/
  protected ModelVector mp_iNeighPredCounts4Mast = new ModelVector(
      "Neighborhood Predation - Counts For Masting?",
      "pr_neighPredCounts4Mast", "pr_npc4mVal", 0, ModelVector.MODEL_ENUM);

	/**Neighborhood seed predation - density of seeds for masting, seeds/m2/yr*/
	protected ModelFloat m_fNeighPredMastingDensity = new ModelFloat(0,
      "Neighborhood Predation - Masting Seed Density, #/m2/yr", 
      "pr_neighPredMastingDensity");
	
	/**Neighborhood seed predation - neighborhood search radius*/
	protected ModelFloat m_fNeighPredRadius = new ModelFloat(0,
      "Neighborhood Predation - Neighbor Search Radius (m)", 
      "pr_neighPredRadius");

	/**Neighborhood seed predation - method for deciding mast */
  protected ModelEnum m_iNeighPredMastDecisionMethod =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"Seed threshold", "Ask disperse"},
                    "Neighborhood Predation - Mast Event Decision Method",
                    "pr_neighPredMastDecisionMethod");
	
	
	
	
	/**Linked neighborhood seed predation - "p0"*/
  protected ModelVector mp_fLnkNeighPredP0 = new ModelVector(
      "(Lnk) Neighborhood Predation - \"p0\"",  "pr_neighPredLnkP0", 
      "pr_npmlp0Val", 0, ModelVector.FLOAT);
  
  /**Linked neighborhood seed predation - min neighbor DBH*/
  protected ModelVector mp_fLnkNeighPredMinDbh = new ModelVector(
      "(Lnk) Neighborhood Predation - Minimum Neighbor DBH (cm)", 
      "pr_neighPredLnkMinNeighDBH", "pr_nplmndVal", 0, ModelVector.FLOAT, true);
  
  /**Linked neighborhood seed predation - neighborhood search radius*/
  protected ModelFloat m_fLnkNeighPredRadius = new ModelFloat(0,
      "(Lnk) Neighborhood Predation - Neighbor Search Radius (m)", 
      "pr_neighPredRadiusLnk");
	
	

  /**Functional response predation - predator initial density in number per 
   * square meter*/
  protected ModelFloat m_fFuncRespPredInitDensity = new ModelFloat(0,
      "Func Resp - Predator Initial Density - num/sq m", "pr_funcRespPredatorInitialDensity");
  
  /**Functional response predation - max instantaneous rate at which predator 
   * abundance declines in the absense of food, in number of predators per 
   * week - season 1*/
  protected ModelFloat m_fFuncRespMaxInstDeclineRate1 = new ModelFloat(0,
      "Func Resp - Max Decline Rate, Season 1, predators/week",
      "pr_funcRespMaxInstDeclineRate1");
  
  /**Functional response predation - max instantaneous rate at which predator 
   * abundance declines in the absense of food, in number of predators per 
   * week - season 2*/
  protected ModelFloat m_fFuncRespMaxInstDeclineRate2 = new ModelFloat(0,
      "Func Resp - Max Decline Rate, Season 2, predators/week",
      "pr_funcRespMaxInstDeclineRate2");
  
  /**Functional response predation - predator population's demographic 
   * efficiency - season 1*/
  protected ModelFloat m_fFuncRespDemogEfficiency1 = new ModelFloat(0,
      "Func Resp - Demographic Efficiency, Season 1", 
      "pr_funcRespDemographicEfficiency1");
  
  /**Functional response predation - predator population's demographic 
   * efficiency - season 2*/
  protected ModelFloat m_fFuncRespDemogEfficiency2 = new ModelFloat(0,
      "Func Resp - Demographic Efficiency, Season 2", 
      "pr_funcRespDemographicEfficiency2");
  
  /**Functional response predation - density-dependent coefficient - season 1*/
  protected ModelFloat m_fFunRespDensDepCoeff1 = new ModelFloat(0,
      "Func Resp - Density Dependent Coeff, Season 1", 
      "pr_funcRespDensityDependentCoefficient1");
  
  /**Functional response predation - density-dependent coefficient - season 2*/
  protected ModelFloat m_fFunRespDensDepCoeff2 = new ModelFloat(0,
      "Func Resp - Density Dependent Coeff, Season 2", 
      "pr_funcRespDensityDependentCoefficient2");
  
  /**Functional response predation - maximum intake rate - number of seeds per
   * predator per day*/
  protected ModelVector mp_fFuncRespMaxIntake = new ModelVector(
      "Func Resp - Max Intake Rate - seeds per predator per day", 
      "pr_funcRespMaxIntakeRate", "pr_frmirVal", 0, ModelVector.FLOAT);
  
  /**Functional response predation - foraging efficiency*/
  protected ModelVector mp_fFuncRespForagingEff = new ModelVector(
      "Func Resp - Foraging Efficiency", "pr_funcRespForagingEfficiency", "pr_frfeVal", 
      0, ModelVector.FLOAT);
  
  /**Functional response predation - proportion of seeds, between 0 and 1, 
   * germinating each week in the germination period*/
  protected ModelFloat m_fFuncRespPropGerm = new ModelFloat(0,
      "Func Resp - Proportion of Seeds Germinating Each Week",
      "pr_funcRespProportionGerminating");

  /**Functional response predation - number of weeks for which seedfall occurs*/
  protected ModelInt m_iFuncRespNumWeeksSeedFall = new ModelInt(0,
      "Func Resp - Number of Weeks in Which Seedfall Occurs", 
      "pr_funcRespNumWeeksSeedFall");
  
  /**Functional response predation - number of weeks to run model - must be 
   * less than or equal to 52*/
  protected ModelInt m_iFuncRespNumWeeksToModel = new ModelInt(0,
      "Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
      "pr_funcRespNumWeeksToModel");
  
  /**Functional response predation - week number in which germination begins*/
  protected ModelInt m_iFuncRespWeekGermStarts = new ModelInt(0,
      "Func Resp - Week Germination Begins",
      "pr_funcRespWeekGerminationStarts");
  
  /**Functional response predation - week number in which season 2 begins*/
  protected ModelInt m_iFuncRespWeekSeason2Starts = new ModelInt(0,
      "Func Resp - Week Season 2 Begins",
      "pr_funcRespWeekSeason2Starts");

  /**Functional response predation - output filename*/
  protected ModelString m_sFuncRespOutputFilename = new ModelString("",
      "Func Resp - Seed Predation Output Filename, If Desired", 
      "pr_funcRespOutputFilename");

  /**Functional response predation - whether or not to preserve predator 
   * densities between model timesteps (true) or re-initialize with predator
   * initial density (false)*/
  protected ModelEnum m_iPreservePredatorDensities = new ModelEnum(
		  new int[] {0, 1}, new String[] {"false", "true"},
      "Func Resp - Keep Predator Densities Between Timesteps",
      "pr_funcRespPreservePredatorDensities");
  
  
  /**Linked functional response predation - predator initial density in number per 
   * square meter*/
  protected ModelFloat m_fLnkFuncRespPredInitDensity = new ModelFloat(0,
      "(Lnk) Func Resp - Predator Initial Density - num/sq m", "pr_funcRespPredatorInitialDensityLnk");
  
  /**Linked functional response predation - max instantaneous rate at which predator 
   * abundance declines in the absense of food, in number of predators per 
   * week - season 1*/
  protected ModelFloat m_fLnkFuncRespMaxInstDeclineRate1 = new ModelFloat(0,
      "(Lnk) Func Resp - Max Decline Rate, Season 1, predators/week",
      "pr_funcRespMaxInstDeclineRate1Lnk");
  
  /**Linked functional response predation - max instantaneous rate at which predator 
   * abundance declines in the absense of food, in number of predators per 
   * week - season 2*/
  protected ModelFloat m_fLnkFuncRespMaxInstDeclineRate2 = new ModelFloat(0,
      "(Lnk) Func Resp - Max Decline Rate, Season 2, predators/week",
      "pr_funcRespMaxInstDeclineRate2Lnk");
  
  /**Linked functional response predation - predator population's demographic 
   * efficiency - season 1*/
  protected ModelFloat m_fLnkFuncRespDemogEfficiency1 = new ModelFloat(0,
      "(Lnk) Func Resp - Demographic Efficiency, Season 1", 
      "pr_funcRespDemographicEfficiency1Lnk");
  
  /**Linked functional response predation - predator population's demographic 
   * efficiency - season 2*/
  protected ModelFloat m_fLnkFuncRespDemogEfficiency2 = new ModelFloat(0,
      "(Lnk) Func Resp - Demographic Efficiency, Season 2", 
      "pr_funcRespDemographicEfficiency2Lnk");
  
  /**Linked functional response predation - density-dependent coefficient - season 1*/
  protected ModelFloat m_fLnkFunRespDensDepCoeff1 = new ModelFloat(0,
      "(Lnk) Func Resp - Density Dependent Coeff, Season 1", 
      "pr_funcRespDensityDependentCoefficient1Lnk");
  
  /**Linked functional response predation - density-dependent coefficient - season 2*/
  protected ModelFloat m_fLnkFunRespDensDepCoeff2 = new ModelFloat(0,
      "(Lnk) Func Resp - Density Dependent Coeff, Season 2", 
      "pr_funcRespDensityDependentCoefficient2Lnk");
  
  /**Linked functional response predation - maximum intake rate - number of seeds per
   * predator per day*/
  protected ModelVector mp_fLnkFuncRespMaxIntake = new ModelVector(
      "(Lnk) Func Resp - Max Intake Rate - seeds per predator per day", 
      "pr_funcRespMaxIntakeRateLnk", "pr_frmirlVal", 0, ModelVector.FLOAT);
  
  /**Linked functional response predation - foraging efficiency*/
  protected ModelVector mp_fLnkFuncRespForagingEff = new ModelVector(
      "(Lnk) Func Resp - Foraging Efficiency", "pr_funcRespForagingEfficiencyLnk", "pr_frfelVal", 
      0, ModelVector.FLOAT);
  
  /**Linked functional response predation - proportion of seeds, between 0 and 1, 
   * germinating each week in the germination period*/
  protected ModelFloat m_fLnkFuncRespPropGerm = new ModelFloat(0,
      "(Lnk) Func Resp - Proportion of Seeds Germinating Each Week",
      "pr_funcRespProportionGerminatingLnk");

  /**Linked functional response predation - number of weeks for which seedfall occurs*/
  protected ModelInt m_iLnkFuncRespNumWeeksSeedFall = new ModelInt(0,
      "(Lnk) Func Resp - Number of Weeks in Which Seedfall Occurs", 
      "pr_funcRespNumWeeksSeedFallLnk");
  
  /**Linked functional response predation - number of weeks to run model - must be 
   * less than or equal to 52*/
  protected ModelInt m_iLnkFuncRespNumWeeksToModel = new ModelInt(0,
      "(Lnk) Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
      "pr_funcRespNumWeeksToModelLnk");
  
  /**Linked functional response predation - week number in which germination begins*/
  protected ModelInt m_iLnkFuncRespWeekGermStarts = new ModelInt(0,
      "(Lnk) Func Resp - Week Germination Begins",
      "pr_funcRespWeekGerminationStartsLnk");
  
  /**Linked functional response predation - week number in which season 2 begins*/
  protected ModelInt m_iLnkFuncRespWeekSeason2Starts = new ModelInt(0,
      "(Lnk) Func Resp - Week Season 2 Begins",
      "pr_funcRespWeekSeason2StartsLnk");

  /**Linked functional response predation - output filename*/
  protected ModelString m_sLnkFuncRespOutputFilename = new ModelString("",
      "(Lnk) Func Resp - Seed Predation Output Filename, If Desired", 
      "pr_funcRespOutputFilenameLnk");

  /**Linked functional response predation - whether or not to preserve predator 
   * densities between model timesteps (true) or re-initialize with predator
   * initial density (false)*/
  protected ModelEnum m_iLnkPreservePredatorDensities = new ModelEnum(
      new int[] {0, 1}, new String[] {"false", "true"},
      "(Lnk) Func Resp - Keep Predator Densities Between Timesteps",
      "pr_funcRespPreservePredatorDensitiesLnk");
  
  
  
    
  

  /**
   * Constructor.  Sets up the child behavior list and adds data appropriately.
   * @param oManager GUIManager object.
   * @throws ModelException passed through from called methods.  Should never
   * be thrown.
   */
  public SeedPredationBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Seed Predation", "seedPredation");

    m_sHelpID = "data.seed_predation_behaviors";
    mp_oChildBehaviors = new Behavior[4];

    //Set up the functional response predation behavior
    mp_oChildBehaviors[0] = new Behavior("funcResponsePredation",
                                         "Functional response seed predation",
                                         "functional response seed predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.ADULT, false);

    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespPredInitDensity);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespMaxInstDeclineRate1);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespMaxInstDeclineRate2);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespDemogEfficiency1);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespDemogEfficiency2);
    mp_oChildBehaviors[0].AddRequiredData(m_fFunRespDensDepCoeff1);
    mp_oChildBehaviors[0].AddRequiredData(m_fFunRespDensDepCoeff2);
    mp_oChildBehaviors[0].AddRequiredData(mp_fFuncRespMaxIntake);
    mp_oChildBehaviors[0].AddRequiredData(mp_fFuncRespForagingEff);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespNumWeeksSeedFall);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespNumWeeksToModel);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespWeekGermStarts);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespWeekSeason2Starts);
    mp_oChildBehaviors[0].AddRequiredData(m_iPreservePredatorDensities);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespPropGerm);
    mp_oChildBehaviors[0].AddRequiredData(m_sFuncRespOutputFilename);
    
    //Set up the neighborhood predation behavior
    mp_oChildBehaviors[1] = new Behavior("Neighborhood Predation",
                                         "Neighborhood Seed Predation",
                                         "Neighborhood Seed Predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.ADULT, false);

    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMastingP0);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredNonMastingP0);
    mp_oChildBehaviors[1].AddRequiredData(mp_iNeighPredCounts4Mast);
    mp_oChildBehaviors[1].AddRequiredData(m_fNeighPredMastingDensity);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMinDbh);
    mp_oChildBehaviors[1].AddRequiredData(m_fNeighPredRadius);
    mp_oChildBehaviors[1].AddRequiredData(m_iNeighPredMastDecisionMethod); 
    
    //Set up the linked functional response predation behavior
    mp_oChildBehaviors[2] = new Behavior("Linked func response predation",
                                  "Functional response seed predation (linked)",
                                    "linked functional response seed predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[2].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[2].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[2].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[2].SetCanApplyTo(TreePopulation.ADULT, false);

    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespPredInitDensity);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespMaxInstDeclineRate1);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespMaxInstDeclineRate2);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespDemogEfficiency1);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespDemogEfficiency2);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFunRespDensDepCoeff1);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFunRespDensDepCoeff2);
    mp_oChildBehaviors[2].AddRequiredData(mp_fLnkFuncRespMaxIntake);
    mp_oChildBehaviors[2].AddRequiredData(mp_fLnkFuncRespForagingEff);
    mp_oChildBehaviors[2].AddRequiredData(m_iLnkFuncRespNumWeeksSeedFall);
    mp_oChildBehaviors[2].AddRequiredData(m_iLnkFuncRespNumWeeksToModel);
    mp_oChildBehaviors[2].AddRequiredData(m_iLnkFuncRespWeekGermStarts);
    mp_oChildBehaviors[2].AddRequiredData(m_iLnkFuncRespWeekSeason2Starts);
    mp_oChildBehaviors[2].AddRequiredData(m_iLnkPreservePredatorDensities);
    mp_oChildBehaviors[2].AddRequiredData(m_fLnkFuncRespPropGerm);
    mp_oChildBehaviors[2].AddRequiredData(m_sLnkFuncRespOutputFilename);
   
    //Set up the neighborhood predation behavior
    mp_oChildBehaviors[3] = new Behavior("Linked Neighborhood Predation",
                                         "Neighborhood Seed Predation (linked)", 
                                         "Linked Neighborhood Seed Predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[3].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[3].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[3].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[3].SetCanApplyTo(TreePopulation.ADULT, false);
    mp_oChildBehaviors[3].AddRequiredData(mp_fLnkNeighPredP0);
    mp_oChildBehaviors[3].AddRequiredData(mp_fLnkNeighPredMinDbh);
    mp_oChildBehaviors[3].AddRequiredData(m_fLnkNeighPredRadius);
    
    //Now set up the all data list
    mp_oAllData.add(m_fFuncRespPredInitDensity);
    mp_oAllData.add(m_fFuncRespMaxInstDeclineRate1);
    mp_oAllData.add(m_fFuncRespMaxInstDeclineRate2);
    mp_oAllData.add(m_fFuncRespDemogEfficiency1);
    mp_oAllData.add(m_fFuncRespDemogEfficiency2);
    mp_oAllData.add(m_fFunRespDensDepCoeff1);
    mp_oAllData.add(m_fFunRespDensDepCoeff2);
    mp_oAllData.add(mp_fFuncRespMaxIntake);
    mp_oAllData.add(mp_fFuncRespForagingEff);
    mp_oAllData.add(m_iFuncRespNumWeeksSeedFall);
    mp_oAllData.add(m_iFuncRespNumWeeksToModel);
    mp_oAllData.add(m_iFuncRespWeekGermStarts);
    mp_oAllData.add(m_iFuncRespWeekSeason2Starts);
    mp_oAllData.add(m_iPreservePredatorDensities);
    mp_oAllData.add(m_fFuncRespPropGerm);
    mp_oAllData.add(m_sFuncRespOutputFilename);
    mp_oAllData.add(mp_fNeighPredMastingP0);
    mp_oAllData.add(mp_fNeighPredNonMastingP0);
    mp_oAllData.add(mp_iNeighPredCounts4Mast);
    mp_oAllData.add(m_fNeighPredMastingDensity);
    mp_oAllData.add(mp_fNeighPredMinDbh);
    mp_oAllData.add(m_fNeighPredRadius);
    mp_oAllData.add(m_iNeighPredMastDecisionMethod); 
    mp_oAllData.add(m_fLnkFuncRespPredInitDensity);
    mp_oAllData.add(m_fLnkFuncRespMaxInstDeclineRate1);
    mp_oAllData.add(m_fLnkFuncRespMaxInstDeclineRate2);
    mp_oAllData.add(m_fLnkFuncRespDemogEfficiency1);
    mp_oAllData.add(m_fLnkFuncRespDemogEfficiency2);
    mp_oAllData.add(m_fLnkFunRespDensDepCoeff1);
    mp_oAllData.add(m_fLnkFunRespDensDepCoeff2);
    mp_oAllData.add(mp_fLnkFuncRespMaxIntake);
    mp_oAllData.add(mp_fLnkFuncRespForagingEff);
    mp_oAllData.add(m_iLnkFuncRespNumWeeksSeedFall);
    mp_oAllData.add(m_iLnkFuncRespNumWeeksToModel);
    mp_oAllData.add(m_iLnkFuncRespWeekGermStarts);
    mp_oAllData.add(m_iLnkFuncRespWeekSeason2Starts);
    mp_oAllData.add(m_iLnkPreservePredatorDensities);
    mp_oAllData.add(m_fLnkFuncRespPropGerm);
    mp_oAllData.add(m_sLnkFuncRespOutputFilename);
    mp_oAllData.add(mp_fLnkNeighPredP0);
    mp_oAllData.add(mp_fLnkNeighPredMinDbh);
    mp_oAllData.add(m_fLnkNeighPredRadius);

    //Make the functional response seed predation grid
    Grid oNewGrid;

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];
    p_oDataMembers[0] = new DataMember("Number of predators", "num preds",
                                       DataMember.FLOAT);
    oNewGrid = new Grid("Seed Predators", p_oDataMembers, null, 8, 8);
    //Add to the functional response behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("funcResponsePredation").AddGrid(oNewGrid);
  }

  /**
   * Sets up neighbor pn values for neighborhood seed predation.
   * @param oPop Tree population
   * @throws ModelException 
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    Behavior oBeh1 = GetBehaviorByKey("Neighborhood Predation"),
             oBeh2 = GetBehaviorByKey("Linked Neighborhood Predation");
    int iNumSpecies = oPop.GetNumberOfSpecies(), i, j;
    boolean bAlreadyPns = false;

    //See if we already had pn values
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor().toLowerCase();
      if (sKey.indexOf("pn") > -1) {
        bAlreadyPns = true;
        break;
      }
    }

    if (bAlreadyPns == false) {

      for (i = 0; i < iNumSpecies; i++) {
        String sSpName = oPop.GetSpeciesNameFromCode(i);
        ModelVector oVector = new ModelVector(
        "Neighborhood Predation - " + sSpName.replace('_', ' ') + 
        " Masting \"pn\"", "pr_neighPredMastingPn" + sSpName, "pr_npmpnVal",
        0, ModelVector.FLOAT);

        mp_oAllData.add(oVector);
        oBeh1.AddRequiredData(oVector);
      }
      
      for (i = 0; i < iNumSpecies; i++) {
        String sSpName = oPop.GetSpeciesNameFromCode(i);
        ModelVector oVector = new ModelVector(
        "Neighborhood Predation - " + sSpName.replace('_', ' ') + 
        " Non-Masting \"pn\"", "pr_neighPredNonMastingPn" + sSpName, 
        "pr_npnmpnVal", 0, ModelVector.FLOAT);

        mp_oAllData.add(oVector);
        oBeh1.AddRequiredData(oVector);
      }
      
      for (i = 0; i < iNumSpecies; i++) {
        String sSpName = oPop.GetSpeciesNameFromCode(i);
        ModelVector oVector = new ModelVector(
        "(Lnk) Neighborhood Predation - " + sSpName.replace('_', ' ') + 
        " \"pn\"", "pr_neighPredPnLnk" + sSpName, 
        "pr_nppnlVal", 0, ModelVector.FLOAT);

        mp_oAllData.add(oVector);
        oBeh2.AddRequiredData(oVector);
      }
    }
    else {
      //We already had pn's.  What we want to do is preserve any for species
      //that didn't change, because they may have values.

      //Create an array of the pn arrays, placed in the new species
      //order
      ModelData[] p_oMastingPns = new ModelData[iNumSpecies],
                  p_oNonMastingPns = new ModelData[iNumSpecies],
                  p_oLinkedPns = new ModelData[iNumSpecies];
      int iCode, iPos1, iPos2;

      for (i = 0; i < iNumSpecies; i++) {
        p_oMastingPns[i] = null;
        p_oNonMastingPns[i] = null;
        p_oLinkedPns[i] = null;
      }

      //Get the existing pn's and put them where they go in our arrays -
      //any for species that were removed will be left behind
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor();
        if (sKey.indexOf(" Masting \"pn\"") > -1) {
          //get the species
          String sSpecies;
          sKey = oData.GetDescriptor();
          iPos1 = sKey.indexOf(" - ") + 3;
          iPos2 = sKey.indexOf(" Masting");
          sSpecies = sKey.substring(iPos1, iPos2).replace(' ', '_');
          iCode = oPop.GetSpeciesCodeFromName(sSpecies);
          if (iCode > -1) {
            p_oMastingPns[iCode] = oData;
          }
        }
      }
      
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor();
        if (sKey.indexOf(" Non-Masting \"pn\"") > -1) {
          //get the species
          String sSpecies;
          sKey = oData.GetDescriptor();
          iPos1 = sKey.indexOf(" - ") + 3;
          iPos2 = sKey.indexOf(" Non-Masting");
          sSpecies = sKey.substring(iPos1, iPos2).replace(' ', '_');
          iCode = oPop.GetSpeciesCodeFromName(sSpecies);
          if (iCode > -1) {
            p_oNonMastingPns[iCode] = oData;
          }
        }
      }
      
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor();
        if (sKey.indexOf("(Lnk)") > -1 && sKey.indexOf("\"pn\"") > -1) {
          
          //get the species
          String sSpecies;
          sKey = oData.GetDescriptor();
          iPos1 = sKey.indexOf(" - ") + 3;
          iPos2 = sKey.indexOf(" \"pn\"");
          sSpecies = sKey.substring(iPos1, iPos2).replace(' ', '_');
          iCode = oPop.GetSpeciesCodeFromName(sSpecies);
          if (iCode > -1) {
            p_oLinkedPns[iCode] = oData;
          }
        }
      }

      //Now remove all pn's from the required data and from all behaviors
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor().toLowerCase();
        if (sKey.indexOf("pn") > -1) {
          mp_oAllData.remove(i);
          i--;
        }
      }
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
          ModelData oData = (ModelData) mp_oChildBehaviors[i].
              mp_oRequiredData.elementAt(j);
          String sKey = oData.GetDescriptor().toLowerCase();
          if (sKey.indexOf("pn") > -1) {
            mp_oChildBehaviors[i].mp_oRequiredData.remove(j);
            j--;
          }
        }
      }

      //Now add back the pn's, creating new ones for any new species
      for (i = 0; i < p_oMastingPns.length; i++) {
        if (p_oMastingPns[i] == null) {
          String sSpName = oPop.GetSpeciesNameFromCode(i);
          p_oMastingPns[i] = new ModelVector(
              "Neighborhood Predation - " + sSpName.replace('_', ' ') + 
              " Masting \"pn\"", "pr_neighPredMastingPn" + sSpName, "pr_npmpnVal",
              0, ModelVector.FLOAT);
        }
        mp_oAllData.add(p_oMastingPns[i]);
        oBeh1.AddRequiredData(p_oMastingPns[i]);
      }  
        
      for (i = 0; i < p_oNonMastingPns.length; i++) {
        if (p_oNonMastingPns[i] == null) {
          String sSpName = oPop.GetSpeciesNameFromCode(i);
          p_oNonMastingPns[i] = new ModelVector(
              "Neighborhood Predation - " + sSpName.replace('_', ' ') + 
              " Non-Masting \"pn\"", "pr_neighPredNonMastingPn" + sSpName, 
              "pr_npnmpnVal", 0, ModelVector.FLOAT);
        }
        mp_oAllData.add(p_oNonMastingPns[i]);
        oBeh1.AddRequiredData(p_oNonMastingPns[i]);
      }
      
      for (i = 0; i < p_oLinkedPns.length; i++) {
        if (p_oLinkedPns[i] == null) {
          String sSpName = oPop.GetSpeciesNameFromCode(i);
          p_oLinkedPns[i] = new ModelVector(
              "(Lnk) Neighborhood Predation - " + sSpName.replace('_', ' ') + 
              " \"pn\"", "pr_neighPredPnLnk" + sSpName, 
              "pr_nppnlVal", 0, ModelVector.FLOAT);
        }
        mp_oAllData.add(p_oLinkedPns[i]);
        oBeh2.AddRequiredData(p_oLinkedPns[i]);
      }
    }
    
  
    //Set up the vector for whether to include in mast 
    if (mp_iNeighPredCounts4Mast.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        ModelEnum oEnum = new ModelEnum(new int[] {
            0, 1}, new String[] {"No", "Yes"}, "", ""); 
        oEnum.SetValue("Yes"); 
        mp_iNeighPredCounts4Mast.GetValue().add(oEnum);
      }
    } else {
      //There were already species - verify that any nulls get their enum
      //added
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iNeighPredCounts4Mast.GetValue().elementAt(i)) {
          mp_iNeighPredCounts4Mast.GetValue().remove(i);
          ModelEnum oEnum = new ModelEnum(new int[] {
              0, 1}, new String[] {"No", "Yes"}, "", ""); 
          oEnum.SetValue("Yes");
          mp_iNeighPredCounts4Mast.GetValue().add(i, oEnum);
        }
      }
    }
    
    //Set up the neighborhood seed predation output grid
    DataMember[] p_oDataMembers = new DataMember[2 * iNumSpecies];
    Grid oNewGrid;

    String sGridName = "Neighborhood Seed Predation";

    //The accessible data members are a seed count for each species and
    //gap status
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i] = new DataMember("Pre-predation seeds for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "startseeds_" + i,
                                         DataMember.FLOAT);
    }
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i + iNumSpecies] = new DataMember("Proportion seeds eaten for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "propeaten_" + i,
                                         DataMember.FLOAT);
    }

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    oNewGrid = m_oManager.AddGrid(oNewGrid);

    //Assign the grid to neighborhood predation  behaviors
    GetBehaviorByKey("Neighborhood Predation").AddGrid(oNewGrid);
    GetBehaviorByKey("Linked Neighborhood Predation").AddGrid(oNewGrid);
  }

  /**
   * Validates the dataset if seed predation is enabled.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException if:
   * <ul>
   * <li>Number of weeks of seedfall is less than or equal to zero</li>
   * <li>Initial predator density is less than or equal to zero</li>
   * <li>Number of weeks to run the model is less than 0 or greater than 52</li>
   * <li>Week to start germination is less than 0 or greater than 52</li>
   * <li>Proportion germinating is not a proportion</li>
   * <li>Neighborhood predation minimum neighbor DBH is less than zero</li>
   * <li>Neighborhood predation neighbor radius is less than zero</li>
   * <li>Neighborhood predation masting seed density threshold is less than
   * zero</li>
   * <li>Either seed predation behavior is present without a disperse behavior
   * also enabled</li>
   * </ul>
   */
  public void ValidateData(TreePopulation oPop) throws javawrapper.
      ModelException {

    if (!AnyBehaviorsEnabled()) {
      return;
    }

    if (GetBehaviorByKey("funcResponsePredation").m_bIsEnabled) {
    	MakeSureGreaterThan(m_iFuncRespNumWeeksSeedFall, 0);
    	MakeSureGreaterThan(m_fFuncRespPredInitDensity, 0);
    	MakeSureGreaterThan(m_iFuncRespNumWeeksToModel, 0);
    	MakeSureGreaterThanEqualTo(m_iFuncRespWeekGermStarts, 0);
      MakeSureIsProportion(m_fFuncRespPropGerm);
      MakeSureLessThan(m_iFuncRespNumWeeksToModel, 53);
      MakeSureLessThan(m_iFuncRespWeekGermStarts, 53);
      MakeSureLessThan(m_iFuncRespWeekSeason2Starts, 53);
      
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
      	throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
      			                     "You must use a Disperse behavior with " +
      			                     GetBehaviorByKey("funcResponsePredation").GetDescriptor()));
      }
    }
    
    if (GetBehaviorByKey("Linked func response predation").m_bIsEnabled) {
      MakeSureGreaterThan(m_iLnkFuncRespNumWeeksSeedFall, 0);
      MakeSureGreaterThan(m_fLnkFuncRespPredInitDensity, 0);
      MakeSureGreaterThan(m_iLnkFuncRespNumWeeksToModel, 0);
      MakeSureGreaterThanEqualTo(m_iLnkFuncRespWeekGermStarts, 0);
      MakeSureIsProportion(m_fLnkFuncRespPropGerm);
      MakeSureLessThan(m_iLnkFuncRespNumWeeksToModel, 53);
      MakeSureLessThan(m_iLnkFuncRespWeekGermStarts, 53);
      MakeSureLessThan(m_iLnkFuncRespWeekSeason2Starts, 53);
      
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                 "You must use a Disperse behavior with " +
                                 GetBehaviorByKey("Linked func response predation").GetDescriptor()));
      }
    }
   
    if (GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled) {
      if(m_iNeighPredMastDecisionMethod.GetValue() == 0)
    	  MakeSureGreaterThan(m_fNeighPredMastingDensity, 0);
    	MakeSureAllNonNegative(mp_fNeighPredMinDbh);
    	MakeSureGreaterThan(m_fNeighPredRadius, 0);
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
      	throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
      			                     "You must use a Disperse behavior with " +
      			                     GetBehaviorByKey("Neighborhood Predation").GetDescriptor()));
      }
    }
    
    if (GetBehaviorByKey("Linked Neighborhood Predation").m_bIsEnabled) {
      MakeSureAllNonNegative(mp_fLnkNeighPredMinDbh);
      MakeSureGreaterThan(m_fLnkNeighPredRadius, 0);
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                 "You must use a Disperse behavior with " +
                                 GetBehaviorByKey("Neighborhood Predation").GetDescriptor()));
      }
      if (!GetBehaviorByKey("Linked func response predation").m_bIsEnabled) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
            "You must use the " + 
            GetBehaviorByKey("Linked func response predation").GetDescriptor() +
            " behavior with " +
            GetBehaviorByKey("Neighborhood Predation").GetDescriptor()));
      }  
    }
  } 
  
  /**
   * Updates the pn neighbor names when a species name is changed.
   * @param sOldSpecies String Old name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   * @param sNewSpecies String New name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   */
  public void ChangeOfSpeciesName (String sOldSpecies, String sNewSpecies) {
    int i;

    sOldSpecies = sOldSpecies.replace('_', ' ');

    //Find the pn's in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sOldSpecies) > -1) {
        if (sKey.indexOf(" Masting \"pn\"") > -1) {
          ModelVector oVector = (ModelVector) oData;
          oVector.m_sDescriptor = "Neighborhood Predation - " + 
              sNewSpecies.replace('_', ' ') + " Masting \"pn\"";
          oVector.m_sXMLTag = "pr_neighPredMastingPn" + sNewSpecies;
        } else if (sKey.indexOf(" Non-Masting \"pn\"") > -1) {
          ModelVector oVector = (ModelVector) oData;
          oVector.m_sDescriptor = "Neighborhood Predation - " + 
              sNewSpecies.replace('_', ' ') + " Non-Masting \"pn\"";
          oVector.m_sXMLTag = "pr_neighPredNonMastingPn" + sNewSpecies;
        } else if (sKey.indexOf("(Lnk") > -1 && 
                   sKey.indexOf("\"pn\"") > -1) {
          ModelVector oVector = (ModelVector) oData;
          oVector.m_sDescriptor = "(Lnk) Neighborhood Predation - " + 
                 sNewSpecies.replace('_', ' ') + " \"pn\"";
          oVector.m_sXMLTag = "pr_neighPredPnLnk" + sNewSpecies;
        }  
      }
    }
  }

  /**
   * Updates the pn when a species is copied. The pn's for that neighbor
   * are made identical to those being copied as well as entries for species
   * within the pn.
   * @param iSpeciesCopyFrom int Species to copy.
   * @param iSpeciesCopyTo int Species that is the copy.
   * @throws ModelException if there is a problem.
   */
  public void CopySpecies (int iSpeciesCopyFrom, int iSpeciesCopyTo) throws ModelException {
    TreePopulation oPop = m_oManager.GetTreePopulation();
    ModelVector oCopyFrom = null, oCopyTo = null;
    ModelData oData;
    Float f1, f2;
    String sCopyFrom = oPop.GetSpeciesNameFromCode(iSpeciesCopyFrom).replace('_', ' '),
           sCopyTo = oPop.GetSpeciesNameFromCode(iSpeciesCopyTo).replace('_', ' ');
    int i;

    //Find the masting pn's in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sCopyFrom) > -1 && sKey.indexOf(" Masting \"pn\"") > -1) {
        oCopyFrom = (ModelVector) oData;
      }
      if (sKey.indexOf(sCopyTo) > -1 && sKey.indexOf(" Masting \"pn\"") > -1) {
        oCopyTo = (ModelVector) oData;
      }
    }

    if (null == oCopyFrom || null == oCopyTo) {
      throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
        "Seed predation could not find neighborhood seed predation pn values " +
        "for neighbor species when copying species."));
    }

    //Do it differently depending on whether there are existing values or not
    //in the target array
    if (oCopyTo.GetValue().size() > 0) {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().remove(i);
        oCopyTo.GetValue().add(i, f2);
      }
    } else {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().add(i, f2);
      }
    }
    
    //Find the non-masting pn's in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sCopyFrom) > -1 && sKey.indexOf(" Non-Masting \"pn\"") > -1) {
        oCopyFrom = (ModelVector) oData;
      }
      if (sKey.indexOf(sCopyTo) > -1 && sKey.indexOf(" Non-Masting \"pn\"") > -1) {
        oCopyTo = (ModelVector) oData;
      }
    }

    if (null == oCopyFrom || null == oCopyTo) {
      throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
        "Seed predation could not find neighborhood seed predation pn values " +
        "for neighbor species when copying species."));
    }

    //Do it differently depending on whether there are existing values or not
    //in the target array
    if (oCopyTo.GetValue().size() > 0) {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().remove(i);
        oCopyTo.GetValue().add(i, f2);
      }
    } else {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().add(i, f2);
      }
    }
    
    //Find the linked pn's in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sCopyFrom) > -1 && sKey.indexOf("\"pn\"") > -1 && 
          sKey.indexOf("(Lnk)") > -1) {
        oCopyFrom = (ModelVector) oData;
      }
      if (sKey.indexOf(sCopyTo) > -1 && sKey.indexOf("\"pn\"") > -1 && 
          sKey.indexOf("(Lnk)") > -1) {
        oCopyTo = (ModelVector) oData;
      }
    }

    if (null == oCopyFrom || null == oCopyTo) {
      throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
        "Seed predation could not find neighborhood seed predation pn values " +
        "for neighbor species when copying species."));
    }

    //Do it differently depending on whether there are existing values or not
    //in the target array
    if (oCopyTo.GetValue().size() > 0) {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().remove(i);
        oCopyTo.GetValue().add(i, f2);
      }
    } else {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().add(i, f2);
      }
    }

    super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);
  }
  
  /**
   * A simple backwards compatibility feature. A parameter was transformed from
   * a single value to a species specific one.  This traps the single value 
   * so parameter files containing it can be read.  This can be eliminated 
   * in release 6.11.    
   * @param sXMLTag XML tag of object to assign.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Ignored.
   * @param oData Data to assign.
   * @return true if assigned, or false if not.
   * @throws ModelException not thrown here.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Attributes oAttributes,
                                        Object oData) throws
      ModelException {
    
    if (sXMLTag.equals("pr_neighPredMinNeighDBHLnk" ) ||
        sXMLTag.equals("pr_neighPredMinNeighDBH" )) {
      
      return true;
    }
    return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag,
                                   oAttributes, oData);
  }
}
