package javawrapper;


import java.io.FileWriter;
import java.io.File;
import java.io.IOException;

/**
 * Tests the SubstrateBehaviors class.
 * <p>
 * Copyright: Copyright (c) Charles D. Canham 2003
 * </p>
 * <p>
 * Company: Institute of Ecosystem Studies
 * </p>
 * 
 * @author Lora E. Murphy
 * @version 1.0
 * 
 * <br>
 * Edit history: <br>
 * ------------------ <br>
 * June 3, 2005: Created (LEM)
 */
public class TestSubstrateBehaviors extends ModelTestCase {

	/**
	 * Tests to make sure old substrate files are properly read.
	 */
	public void testBackwardsCompatibility() {
		GUIManager oManager = null;
		String sFileName = null;
		try {

			oManager = new GUIManager(null);

			// Load an old file
			oManager.ClearCurrentData();
			sFileName = WriteXMLOldFile1();
			oManager.InputXMLParameterFile(sFileName);
			SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

			// Verify that all the values are set correctly
			assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1E-4, 0.001);
			assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
			assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0E-4, 0.001);
			assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
			assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
			assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
			assertEquals(oSubs.m_fDecayedlogA.GetValue(), -0.7985, 0.001);
			assertEquals(oSubs.m_fDecayedlogB.GetValue(), 1, 0.001);
			assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.18, 0.001);
			assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17, 0.001);
			assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
			assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
			assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.3, 0.001);
			assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
			assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
			assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.35, 0.001);
			assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
			assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06, 0.001);
			assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
			assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.16, 0.001);
			assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
			assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
			assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
			assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.01, 0.001);
			assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
			assertEquals(oSubs.m_fMossProportion.GetValue(), 0, 0.001);
			assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 0);
			assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 50);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(0)).floatValue(), 0.7, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(1)).floatValue(), 0.6, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.5, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.4, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.3, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.2, 0.001);

			// Load an old file
			oManager.ClearCurrentData();
			sFileName = WriteXMLOldFile2();
			oManager.InputXMLParameterFile(sFileName);
			oSubs = oManager.GetSubstrateBehaviors();

			// Verify that all the values are set correctly
			assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1, 0.001);
			assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
			assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0, 0.001);
			assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
			assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
			assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
			assertEquals(oSubs.m_fDecayedlogA.GetValue(), -1.38629, 0.001);
			assertEquals(oSubs.m_fDecayedlogB.GetValue(), 1, 0.001);
			assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.23, 0.001);
			assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17, 0.001);
			assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
			assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
			assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.2, 0.001);
			assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
			assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
			assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.32, 0.001);
			assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
			assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06, 0.001);
			assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
			assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.13, 0.001);
			assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
			assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
			assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
			assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.02, 0.001);
			assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
			assertEquals(oSubs.m_fMossProportion.GetValue(), 0.7, 0.001);
			assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 1);
			assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 3);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(0)).floatValue(), 1, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(1)).floatValue(), 0.2, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.18, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.4, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.8, 0.001);
		} catch (ModelException oErr) {
			fail("Substrate backwards compatibility failed with message "
					+ oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			if (sFileName != null) {
				new File(sFileName).delete();
			}
		}
	}

	/**
	 * Tests to make sure the appropriate parameters are displayed for each
	 * behavior.
	 */
	public void testFormatDataForDisplay() {
		GUIManager oManager = null;
		String sFileName = null;
		try {
			oManager = new GUIManager(null);
			oManager.ClearCurrentData();
			sFileName = WriteNeutralFile();
			oManager.InputXMLParameterFile(sFileName);
			TreePopulation oPop = oManager.GetTreePopulation();
			SubstrateBehaviors oSubstrate = oManager.GetSubstrateBehaviors();
			String[] p_sExpected;
			Behavior oBeh;

			// Case: no substrate behaviors are enabled.
			assertEquals(null, oSubstrate.FormatDataForDisplay(oPop));

			// Case: only regular substrate is enabled.
			oManager.ClearCurrentData();
			sFileName = WriteNeutralFile();
			oManager.InputXMLParameterFile(sFileName);
			oPop = oManager.GetTreePopulation();
			oSubstrate = oManager.GetSubstrateBehaviors();
			oBeh = oSubstrate.GetBehaviorByKey("substrate");
			oBeh.m_bIsEnabled = true;
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
			p_sExpected = new String[] { "Proportion of Dead that Fall",
					"Proportion of Fallen that Uproot",
					"Proportion of Snags that Uproot",
					"Scarified Soil Annual Decay Alpha",
					"Scarified Soil Annual Decay Beta",
					"Tip-Up Mounds Annual Decay Alpha",
					"Tip-Up Mounds Annual Decay Beta", "Fresh Log Annual Decay Alpha",
					"Fresh Log Annual Decay Beta", "Decayed Log Annual Decay Alpha",
					"Decayed Log Annual Decay Beta",
					"Initial Conditions Proportion of Scarified Soil",
					"Partial Cut Proportion of Scarified Soil",
					"Gap Cut Proportion of Scarified Soil",
					"Clear Cut Proportion of Scarified Soil",
					"Initial Conditions Proportion of Tip-Up Mounds",
					"Partial Cut Proportion of Tip-Up Mounds",
					"Gap Cut Proportion of Tip-Up Mounds",
					"Clear Cut Proportion of Tip-Up Mounds",
					"Initial Conditions Proportion of Fresh Logs",
					"Partial Cut Proportion of Fresh Logs",
					"Gap Cut Proportion of Fresh Logs",
					"Clear Cut Proportion of Fresh Logs",
					"Initial Conditions Proportion of Decayed Logs",
					"Partial Cut Proportion of Decayed Logs",
					"Gap Cut Proportion of Decayed Logs",
					"Clear Cut Proportion of Decayed Logs",
					"Uprooted Tree Radius Increase Factor for Root Rip-Out",
					"Proportion of Forest Floor Litter/Moss Pool that is Moss",
					"Use Directional Tree Fall",
					"Maximum Number of Years that Decay Occurs" };
			TestTable(oSubstrate.FormatDataForDisplay(oPop), p_sExpected);

			// Case: only detailed substrate is enabled.
			oManager.ClearCurrentData();
			sFileName = WriteNeutralFile();
			oManager.InputXMLParameterFile(sFileName);
			oPop = oManager.GetTreePopulation();
			oSubstrate = oManager.GetSubstrateBehaviors();
			oBeh = oSubstrate.GetBehaviorByKey("Detailed Substrate");
			oBeh.m_bIsEnabled = true;
			oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
			p_sExpected = new String[] { "Initial Small Logs Mean Diameter (cm)",
					"Initial Large Logs Mean Diameter (cm)",
					"Partial Cut Small Logs Mean Diameter (cm)",
					"Partial Cut Large Logs Mean Diameter (cm)",
					"Gap Cut Small Logs Mean Diameter (cm)",
					"Gap Cut Large Logs Mean Diameter (cm)",
					"Clear Cut Small Logs Mean Diameter (cm)",
					"Clear Cut Large Logs Mean Diameter (cm)",
					"Prop. Live Trees Entering Decay Class 1 (0-1)",
					"Prop. Live Trees Entering Decay Class 2 (0-1)",
					"Prop. Live Trees Entering Decay Class 3 (0-1)",
					"Prop. Live Trees Entering Decay Class 4 (0-1)",
					"Prop. Live Trees Entering Decay Class 5 (0-1)",
					"Prop. Snags Entering Decay Class 1 (0-1)",
					"Prop. Snags Entering Decay Class 2 (0-1)",
					"Prop. Snags Entering Decay Class 3 (0-1)",
					"Prop. Snags Entering Decay Class 4 (0-1)",
					"Prop. Snags Entering Decay Class 5 (0-1)", "Species Group",
					"Species Group 1 Small Class 1 Log Decay Alpha",
					"Species Group 1 Small Class 2 Log Decay Alpha",
					"Species Group 1 Small Class 3 Log Decay Alpha",
					"Species Group 1 Small Class 4 Log Decay Alpha",
					"Species Group 1 Small Class 5 Log Decay Alpha",
					"Species Group 1 Large Class 1 Log Decay Alpha",
					"Species Group 1 Large Class 2 Log Decay Alpha",
					"Species Group 1 Large Class 3 Log Decay Alpha",
					"Species Group 1 Large Class 4 Log Decay Alpha",
					"Species Group 1 Large Class 5 Log Decay Alpha",
					"Species Group 2 Small Class 1 Log Decay Alpha",
					"Species Group 2 Small Class 2 Log Decay Alpha",
					"Species Group 2 Small Class 3 Log Decay Alpha",
					"Species Group 2 Small Class 4 Log Decay Alpha",
					"Species Group 2 Small Class 5 Log Decay Alpha",
					"Species Group 2 Large Class 1 Log Decay Alpha",
					"Species Group 2 Large Class 2 Log Decay Alpha",
					"Species Group 2 Large Class 3 Log Decay Alpha",
					"Species Group 2 Large Class 4 Log Decay Alpha",
					"Species Group 2 Large Class 5 Log Decay Alpha",
					"Species Group 3 Small Class 1 Log Decay Alpha",
					"Species Group 3 Small Class 2 Log Decay Alpha",
					"Species Group 3 Small Class 3 Log Decay Alpha",
					"Species Group 3 Small Class 4 Log Decay Alpha",
					"Species Group 3 Small Class 5 Log Decay Alpha",
					"Species Group 3 Large Class 1 Log Decay Alpha",
					"Species Group 3 Large Class 2 Log Decay Alpha",
					"Species Group 3 Large Class 3 Log Decay Alpha",
					"Species Group 3 Large Class 4 Log Decay Alpha",
					"Species Group 3 Large Class 5 Log Decay Alpha",
					"Species Group 1 Small Class 1 Log Decay Beta",
					"Species Group 1 Small Class 2 Log Decay Beta",
					"Species Group 1 Small Class 3 Log Decay Beta",
					"Species Group 1 Small Class 4 Log Decay Beta",
					"Species Group 1 Small Class 5 Log Decay Beta",
					"Species Group 1 Large Class 1 Log Decay Beta",
					"Species Group 1 Large Class 2 Log Decay Beta",
					"Species Group 1 Large Class 3 Log Decay Beta",
					"Species Group 1 Large Class 4 Log Decay Beta",
					"Species Group 1 Large Class 5 Log Decay Beta",
					"Species Group 2 Small Class 1 Log Decay Beta",
					"Species Group 2 Small Class 2 Log Decay Beta",
					"Species Group 2 Small Class 3 Log Decay Beta",
					"Species Group 2 Small Class 4 Log Decay Beta",
					"Species Group 2 Small Class 5 Log Decay Beta",
					"Species Group 2 Large Class 1 Log Decay Beta",
					"Species Group 2 Large Class 2 Log Decay Beta",
					"Species Group 2 Large Class 3 Log Decay Beta",
					"Species Group 2 Large Class 4 Log Decay Beta",
					"Species Group 2 Large Class 5 Log Decay Beta",
					"Species Group 3 Small Class 1 Log Decay Beta",
					"Species Group 3 Small Class 2 Log Decay Beta",
					"Species Group 3 Small Class 3 Log Decay Beta",
					"Species Group 3 Small Class 4 Log Decay Beta",
					"Species Group 3 Small Class 5 Log Decay Beta",
					"Species Group 3 Large Class 1 Log Decay Beta",
					"Species Group 3 Large Class 2 Log Decay Beta",
					"Species Group 3 Large Class 3 Log Decay Beta",
					"Species Group 3 Large Class 4 Log Decay Beta",
					"Species Group 3 Large Class 5 Log Decay Beta",
					"Boundary Between Log Diam Classes (cm)",
					"Proportion of Fallen that Uproot",
					"Proportion of Snags that Uproot",
					"Scarified Soil Annual Decay Alpha",
					"Tip-Up Mounds Annual Decay Alpha",
					"Scarified Soil Annual Decay Beta",
					"Tip-Up Mounds Annual Decay Beta",
					"Initial Conditions Proportion of Scarified Soil",
					"Initial Conditions Proportion of Tip-Up Mounds",
					"Species Group 1 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 5 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 5 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 5 Initial Log Prop (0-1)",
					"Partial Cut Proportion of Scarified Soil",
					"Partial Cut Proportion of Tip-Up Mounds",
					"Species Group 1 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 5 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 5 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 5 Partial Cut Log (0-1)",
					"Gap Cut Proportion of Scarified Soil",
					"Gap Cut Proportion of Tip-Up Mounds",
					"Species Group 1 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 5 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 5 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 5 Gap Cut Log (0-1)",
					"Clear Cut Proportion of Scarified Soil",
					"Clear Cut Proportion of Tip-Up Mounds",
					"Species Group 1 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 5 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 5 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 5 Clear Cut Log (0-1)",
					"Uprooted Tree Radius Increase Factor for Root Rip-Out",
					"Proportion of Forest Floor Litter/Moss Pool that is Moss",
					"Maximum Number of Years that Decay Occurs",
					"Use Directional Tree Fall" };
			TestTable(oSubstrate.FormatDataForDisplay(oPop), p_sExpected);

			// Case: all are enabled.
			oManager.ClearCurrentData();
			sFileName = WriteNeutralFile();
			oManager.InputXMLParameterFile(sFileName);
			oPop = oManager.GetTreePopulation();
			oSubstrate = oManager.GetSubstrateBehaviors();
			int i;
			for (i = 0; i < oSubstrate.mp_oChildBehaviors.length; i++) {
				oSubstrate.mp_oChildBehaviors[i].m_bIsEnabled = true;
				if (oSubstrate.mp_oChildBehaviors[i].GetKey().indexOf("Storm Killed") == -1)
					oSubstrate.mp_oChildBehaviors[i]
							.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
				else
					oSubstrate.mp_oChildBehaviors[i]
							.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SNAG,
									oPop));
			}
			p_sExpected = new String[] { "Proportion of Dead that Fall",
					"Proportion of Fallen that Uproot",
					"Proportion of Snags that Uproot",
					"Scarified Soil Annual Decay Alpha",
					"Scarified Soil Annual Decay Beta",
					"Tip-Up Mounds Annual Decay Alpha",
					"Tip-Up Mounds Annual Decay Beta", "Fresh Log Annual Decay Alpha",
					"Fresh Log Annual Decay Beta", "Decayed Log Annual Decay Alpha",
					"Decayed Log Annual Decay Beta",
					"Initial Conditions Proportion of Scarified Soil",
					"Partial Cut Proportion of Scarified Soil",
					"Gap Cut Proportion of Scarified Soil",
					"Clear Cut Proportion of Scarified Soil",
					"Initial Conditions Proportion of Tip-Up Mounds",
					"Partial Cut Proportion of Tip-Up Mounds",
					"Gap Cut Proportion of Tip-Up Mounds",
					"Clear Cut Proportion of Tip-Up Mounds",
					"Initial Conditions Proportion of Fresh Logs",
					"Partial Cut Proportion of Fresh Logs",
					"Gap Cut Proportion of Fresh Logs",
					"Clear Cut Proportion of Fresh Logs",
					"Initial Conditions Proportion of Decayed Logs",
					"Partial Cut Proportion of Decayed Logs",
					"Gap Cut Proportion of Decayed Logs",
					"Clear Cut Proportion of Decayed Logs",
					"Uprooted Tree Radius Increase Factor for Root Rip-Out",
					"Proportion of Forest Floor Litter/Moss Pool that is Moss",
					"Use Directional Tree Fall",
					"Maximum Number of Years that Decay Occurs",
					"Initial Small Logs Mean Diameter (cm)",
					"Initial Large Logs Mean Diameter (cm)",
					"Partial Cut Small Logs Mean Diameter (cm)",
					"Partial Cut Large Logs Mean Diameter (cm)",
					"Gap Cut Small Logs Mean Diameter (cm)",
					"Gap Cut Large Logs Mean Diameter (cm)",
					"Clear Cut Small Logs Mean Diameter (cm)",
					"Clear Cut Large Logs Mean Diameter (cm)",
					"Prop. Live Trees Entering Decay Class 1 (0-1)",
					"Prop. Live Trees Entering Decay Class 2 (0-1)",
					"Prop. Live Trees Entering Decay Class 3 (0-1)",
					"Prop. Live Trees Entering Decay Class 4 (0-1)",
					"Prop. Live Trees Entering Decay Class 5 (0-1)",
					"Prop. Snags Entering Decay Class 1 (0-1)",
					"Prop. Snags Entering Decay Class 2 (0-1)",
					"Prop. Snags Entering Decay Class 3 (0-1)",
					"Prop. Snags Entering Decay Class 4 (0-1)",
					"Prop. Snags Entering Decay Class 5 (0-1)", "Species Group",
					"Species Group 1 Small Class 1 Log Decay Alpha",
					"Species Group 1 Small Class 2 Log Decay Alpha",
					"Species Group 1 Small Class 3 Log Decay Alpha",
					"Species Group 1 Small Class 4 Log Decay Alpha",
					"Species Group 1 Small Class 5 Log Decay Alpha",
					"Species Group 1 Large Class 1 Log Decay Alpha",
					"Species Group 1 Large Class 2 Log Decay Alpha",
					"Species Group 1 Large Class 3 Log Decay Alpha",
					"Species Group 1 Large Class 4 Log Decay Alpha",
					"Species Group 1 Large Class 5 Log Decay Alpha",
					"Species Group 2 Small Class 1 Log Decay Alpha",
					"Species Group 2 Small Class 2 Log Decay Alpha",
					"Species Group 2 Small Class 3 Log Decay Alpha",
					"Species Group 2 Small Class 4 Log Decay Alpha",
					"Species Group 2 Small Class 5 Log Decay Alpha",
					"Species Group 2 Large Class 1 Log Decay Alpha",
					"Species Group 2 Large Class 2 Log Decay Alpha",
					"Species Group 2 Large Class 3 Log Decay Alpha",
					"Species Group 2 Large Class 4 Log Decay Alpha",
					"Species Group 2 Large Class 5 Log Decay Alpha",
					"Species Group 3 Small Class 1 Log Decay Alpha",
					"Species Group 3 Small Class 2 Log Decay Alpha",
					"Species Group 3 Small Class 3 Log Decay Alpha",
					"Species Group 3 Small Class 4 Log Decay Alpha",
					"Species Group 3 Small Class 5 Log Decay Alpha",
					"Species Group 3 Large Class 1 Log Decay Alpha",
					"Species Group 3 Large Class 2 Log Decay Alpha",
					"Species Group 3 Large Class 3 Log Decay Alpha",
					"Species Group 3 Large Class 4 Log Decay Alpha",
					"Species Group 3 Large Class 5 Log Decay Alpha",
					"Species Group 1 Small Class 1 Log Decay Beta",
					"Species Group 1 Small Class 2 Log Decay Beta",
					"Species Group 1 Small Class 3 Log Decay Beta",
					"Species Group 1 Small Class 4 Log Decay Beta",
					"Species Group 1 Small Class 5 Log Decay Beta",
					"Species Group 1 Large Class 1 Log Decay Beta",
					"Species Group 1 Large Class 2 Log Decay Beta",
					"Species Group 1 Large Class 3 Log Decay Beta",
					"Species Group 1 Large Class 4 Log Decay Beta",
					"Species Group 1 Large Class 5 Log Decay Beta",
					"Species Group 2 Small Class 1 Log Decay Beta",
					"Species Group 2 Small Class 2 Log Decay Beta",
					"Species Group 2 Small Class 3 Log Decay Beta",
					"Species Group 2 Small Class 4 Log Decay Beta",
					"Species Group 2 Small Class 5 Log Decay Beta",
					"Species Group 2 Large Class 1 Log Decay Beta",
					"Species Group 2 Large Class 2 Log Decay Beta",
					"Species Group 2 Large Class 3 Log Decay Beta",
					"Species Group 2 Large Class 4 Log Decay Beta",
					"Species Group 2 Large Class 5 Log Decay Beta",
					"Species Group 3 Small Class 1 Log Decay Beta",
					"Species Group 3 Small Class 2 Log Decay Beta",
					"Species Group 3 Small Class 3 Log Decay Beta",
					"Species Group 3 Small Class 4 Log Decay Beta",
					"Species Group 3 Small Class 5 Log Decay Beta",
					"Species Group 3 Large Class 1 Log Decay Beta",
					"Species Group 3 Large Class 2 Log Decay Beta",
					"Species Group 3 Large Class 3 Log Decay Beta",
					"Species Group 3 Large Class 4 Log Decay Beta",
					"Species Group 3 Large Class 5 Log Decay Beta",
					"Boundary Between Log Diam Classes (cm)",
					"Species Group 1 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 1 Large Class 5 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 2 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 2 Large Class 5 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 1 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 2 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 3 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 4 Initial Log Prop (0-1)",
					"Species Group 3 Small Class 5 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 1 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 2 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 3 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 4 Initial Log Prop (0-1)",
					"Species Group 3 Large Class 5 Initial Log Prop (0-1)",
					"Species Group 1 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 1 Large Class 5 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 2 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 2 Large Class 5 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 1 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 2 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 3 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 4 Partial Cut Log (0-1)",
					"Species Group 3 Small Class 5 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 1 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 2 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 3 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 4 Partial Cut Log (0-1)",
					"Species Group 3 Large Class 5 Partial Cut Log (0-1)",
					"Species Group 1 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 1 Large Class 5 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 2 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 2 Large Class 5 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 1 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 2 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 3 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 4 Gap Cut Log (0-1)",
					"Species Group 3 Small Class 5 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 1 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 2 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 3 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 4 Gap Cut Log (0-1)",
					"Species Group 3 Large Class 5 Gap Cut Log (0-1)",
					"Species Group 1 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 1 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 1 Large Class 5 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 2 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 2 Large Class 5 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 1 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 2 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 3 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 4 Clear Cut Log (0-1)",
					"Species Group 3 Small Class 5 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 1 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 2 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 3 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 4 Clear Cut Log (0-1)",
					"Species Group 3 Large Class 5 Clear Cut Log (0-1)" };

			TestTable(oSubstrate.FormatDataForDisplay(oPop), p_sExpected);

			System.out.println("FormatDataForDisplay succeeded for Substrate.");
		} catch (ModelException oErr) {
			fail("Caught error with message " + oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		}
	}

	/**
	 * Tests to make sure parameter files are properly read.
	 */
	public void testParFileRead() {
		GUIManager oManager = null;
		String sFileName = null;
		try {

			oManager = new GUIManager(null);

			// ***********************
			// Test backwards compatibility
			// ***********************
			oManager.ClearCurrentData();
			sFileName = WriteXMLValidFile1();
			oManager.InputXMLParameterFile(sFileName);
			SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

			// Verify that all the values are set correctly
			assertEquals(oSubs.m_fScarSoilA.GetValue(), -5.1, 0.001);
			assertEquals(oSubs.m_fScarSoilB.GetValue(), 4.4, 0.001);
			assertEquals(oSubs.m_fTipUpA.GetValue(), -7.0, 0.001);
			assertEquals(oSubs.m_fTipUpB.GetValue(), 4.3, 0.001);
			assertEquals(oSubs.m_fFreshlogA.GetValue(), -0.05, 0.001);
			assertEquals(oSubs.m_fFreshlogB.GetValue(), 6.5, 0.001);
			assertEquals(oSubs.m_fDecayedlogA.GetValue(), -0.2, 0.001);
			assertEquals(oSubs.m_fDecayedlogB.GetValue(), 5.2, 0.001);
			assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.23, 0.001);
			assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.17, 0.001);
			assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.45, 0.001);
			assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.36, 0.001);
			assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.2, 0.001);
			assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.15, 0.001);
			assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.34, 0.001);
			assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.32, 0.001);
			assertEquals(oSubs.m_fInitCondFreshLog.GetValue(), 0.01, 0.001);
			assertEquals(oSubs.m_fPartialCutFreshLog.GetValue(), 0.06, 0.001);
			assertEquals(oSubs.m_fGapCutFreshLog.GetValue(), 0.09, 0.001);
			assertEquals(oSubs.m_fClearCutFreshLog.GetValue(), 0.13, 0.001);
			assertEquals(oSubs.m_fInitCondDecLog.GetValue(), 0.11, 0.001);
			assertEquals(oSubs.m_fPartialCutDecLog.GetValue(), 0.04, 0.001);
			assertEquals(oSubs.m_fGapCutDecLog.GetValue(), 0.12, 0.001);
			assertEquals(oSubs.m_fClearCutDecLog.GetValue(), 0.02, 0.001);
			assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3.1, 0.001);
			assertEquals(oSubs.m_fMossProportion.GetValue(), 0.7, 0.001);
			assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 1);
			assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 3);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(0)).floatValue(), 1, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfDeadThatFall.GetValue()
					.elementAt(1)).floatValue(), 0.2, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.18, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.4, 0.001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.8, 0.001);
		} catch (ModelException oErr) {
			fail("Substrate backwards compatibility failed with message "
					+ oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			if (sFileName != null) {
				new File(sFileName).delete();
			}
		}

		try {
			// ***********************
			// Test detailed substrate reading
			// ***********************

			oManager.ClearCurrentData();
			sFileName = WriteXMLValidFile2();
			oManager.InputXMLParameterFile(sFileName);
			SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

			assertEquals(oSubs.m_fLogSizeClassBoundary.GetValue(), 40, 0.0001);
			ModelEnum oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(0);
			assertEquals(oEnum.GetValue(), 1);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(1);
			assertEquals(oEnum.GetValue(), 2);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(2);
			assertEquals(oEnum.GetValue(), 3);
			assertEquals(oSubs.m_fPropFallenTreesDecayClass1.GetValue(), 0.1, 0.0001);
			assertEquals(oSubs.m_fPropFallenTreesDecayClass2.GetValue(), 0.2, 0.0001);
			assertEquals(oSubs.m_fPropFallenTreesDecayClass3.GetValue(), 0.3, 0.0001);
			assertEquals(oSubs.m_fPropFallenTreesDecayClass4.GetValue(), 0.04, 0.0001);
			assertEquals(oSubs.m_fPropFallenTreesDecayClass5.GetValue(), 0.36, 0.0001);
			assertEquals(oSubs.m_fPropFallenSnagsDecayClass1.GetValue(), 0.006,	0.0001);
			assertEquals(oSubs.m_fPropFallenSnagsDecayClass2.GetValue(), 0.007,	0.0001);
			assertEquals(oSubs.m_fPropFallenSnagsDecayClass3.GetValue(), 0.008,	0.0001);
			assertEquals(oSubs.m_fPropFallenSnagsDecayClass4.GetValue(), 0.009, 0.0001);
			assertEquals(oSubs.m_fPropFallenSnagsDecayClass5.GetValue(), 0.97, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1Alpha.GetValue(), -1, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2Alpha.GetValue(), -0.8, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3Alpha.GetValue(), -1.2, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4Alpha.GetValue(), -0.5, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5Alpha.GetValue(), -2, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1Alpha.GetValue(), -3, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2Alpha.GetValue(), -4, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3Alpha.GetValue(), -5, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4Alpha.GetValue(), -6, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5Alpha.GetValue(), -7, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1Beta.GetValue(), -8, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2Beta.GetValue(), -9, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3Beta.GetValue(), -10, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4Beta.GetValue(), -11, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5Beta.GetValue(), -12, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1Beta.GetValue(), -13, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2Beta.GetValue(), -14, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3Beta.GetValue(), -15, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4Beta.GetValue(), -16, 0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5Beta.GetValue(), -17, 0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1InitLog.GetValue(), 0.0011,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2InitLog.GetValue(), 0.0012,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3InitLog.GetValue(), 0.0013,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4InitLog.GetValue(), 0.0014,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5InitLog.GetValue(), 0.0015,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1InitLog.GetValue(), 0.0016,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2InitLog.GetValue(), 0.0017,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3InitLog.GetValue(), 0.0018,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4InitLog.GetValue(), 0.0019,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5InitLog.GetValue(), 0.002,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1PartCutLog.GetValue(), 0.0021,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2PartCutLog.GetValue(), 0.0022,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3PartCutLog.GetValue(), 0.0023,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4PartCutLog.GetValue(), 0.0024,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5PartCutLog.GetValue(), 0.0025,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1PartCutLog.GetValue(), 0.0026,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2PartCutLog.GetValue(), 0.0027,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3PartCutLog.GetValue(), 0.0028,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4PartCutLog.GetValue(), 0.0029,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5PartCutLog.GetValue(), 0.0030,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1GapCutLog.GetValue(), 0.0031,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2GapCutLog.GetValue(), 0.0032,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3GapCutLog.GetValue(), 0.0033,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4GapCutLog.GetValue(), 0.0034,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5GapCutLog.GetValue(), 0.0035,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1GapCutLog.GetValue(), 0.0036,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2GapCutLog.GetValue(), 0.0037,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3GapCutLog.GetValue(), 0.0038,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4GapCutLog.GetValue(), 0.0039,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5GapCutLog.GetValue(), 0.0040,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass1ClearCutLog.GetValue(), 0.0041,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass2ClearCutLog.GetValue(), 0.0042,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass3ClearCutLog.GetValue(), 0.0043,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass4ClearCutLog.GetValue(), 0.0044,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1SmallClass5ClearCutLog.GetValue(), 0.0045,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass1ClearCutLog.GetValue(), 0.0046,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass2ClearCutLog.GetValue(), 0.0047,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass3ClearCutLog.GetValue(), 0.0048,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass4ClearCutLog.GetValue(), 0.0049,
					0.0001);
			assertEquals(oSubs.m_fSpGroup1LargeClass5ClearCutLog.GetValue(), 0.0050,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1Alpha.GetValue(), -18, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2Alpha.GetValue(), -19, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3Alpha.GetValue(), -20, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4Alpha.GetValue(), -21, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5Alpha.GetValue(), -22, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1Alpha.GetValue(), -23, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2Alpha.GetValue(), -24, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3Alpha.GetValue(), -25, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4Alpha.GetValue(), -26, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5Alpha.GetValue(), -27, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1Beta.GetValue(), 0.012, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2Beta.GetValue(), 0.013, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3Beta.GetValue(), 0.014, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4Beta.GetValue(), 0.015, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5Beta.GetValue(), 0.016, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1Beta.GetValue(), 0.017, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2Beta.GetValue(), 0.018, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3Beta.GetValue(), 0.019, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4Beta.GetValue(), 0.020, 0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5Beta.GetValue(), 0.021, 0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1InitLog.GetValue(), 0.0051,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2InitLog.GetValue(), 0.0052,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3InitLog.GetValue(), 0.0053,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4InitLog.GetValue(), 0.0054,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5InitLog.GetValue(), 0.0055,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1InitLog.GetValue(), 0.0056,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2InitLog.GetValue(), 0.0057,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3InitLog.GetValue(), 0.0058,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4InitLog.GetValue(), 0.0059,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5InitLog.GetValue(), 0.0060,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1PartCutLog.GetValue(), 0.0061,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2PartCutLog.GetValue(), 0.0062,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3PartCutLog.GetValue(), 0.0063,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4PartCutLog.GetValue(), 0.0064,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5PartCutLog.GetValue(), 0.0065,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1PartCutLog.GetValue(), 0.0066,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2PartCutLog.GetValue(), 0.0067,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3PartCutLog.GetValue(), 0.0068,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4PartCutLog.GetValue(), 0.0069,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5PartCutLog.GetValue(), 0.0070,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1GapCutLog.GetValue(), 0.0071,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2GapCutLog.GetValue(), 0.0072,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3GapCutLog.GetValue(), 0.0073,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4GapCutLog.GetValue(), 0.0074,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5GapCutLog.GetValue(), 0.0075,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1GapCutLog.GetValue(), 0.0076,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2GapCutLog.GetValue(), 0.0077,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3GapCutLog.GetValue(), 0.0078,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4GapCutLog.GetValue(), 0.0079,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5GapCutLog.GetValue(), 0.0080,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass1ClearCutLog.GetValue(), 0.0081,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass2ClearCutLog.GetValue(), 0.0082,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass3ClearCutLog.GetValue(), 0.0083,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass4ClearCutLog.GetValue(), 0.0084,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2SmallClass5ClearCutLog.GetValue(), 0.0085,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass1ClearCutLog.GetValue(), 0.0086,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass2ClearCutLog.GetValue(), 0.0087,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass3ClearCutLog.GetValue(), 0.0088,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass4ClearCutLog.GetValue(), 0.0089,
					0.0001);
			assertEquals(oSubs.m_fSpGroup2LargeClass5ClearCutLog.GetValue(), 0.0090,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1Alpha.GetValue(), -0.1, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2Alpha.GetValue(), -0.2, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3Alpha.GetValue(), -0.3, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4Alpha.GetValue(), -0.4, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5Alpha.GetValue(), -0.5, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1Alpha.GetValue(), -0.6, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2Alpha.GetValue(), -0.7, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3Alpha.GetValue(), -0.8, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4Alpha.GetValue(), -0.9, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5Alpha.GetValue(), -0.11, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1Beta.GetValue(), -0.12, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2Beta.GetValue(), -0.13, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3Beta.GetValue(), -0.14, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4Beta.GetValue(), -0.15, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5Beta.GetValue(), -0.16, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1Beta.GetValue(), -0.17, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2Beta.GetValue(), -0.18, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3Beta.GetValue(), -0.19, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4Beta.GetValue(), -0.21, 0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5Beta.GetValue(), -0.22, 0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1InitLog.GetValue(), 0.0091,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2InitLog.GetValue(), 0.0092,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3InitLog.GetValue(), 0.0093,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4InitLog.GetValue(), 0.0094,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5InitLog.GetValue(), 0.0095,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1InitLog.GetValue(), 0.0096,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2InitLog.GetValue(), 0.0097,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3InitLog.GetValue(), 0.0098,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4InitLog.GetValue(), 0.0099,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5InitLog.GetValue(), 0.0001,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1PartCutLog.GetValue(), 0.0002,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2PartCutLog.GetValue(), 0.0003,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3PartCutLog.GetValue(), 0.0004,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4PartCutLog.GetValue(), 0.0005,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5PartCutLog.GetValue(), 0.0006,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1PartCutLog.GetValue(), 0.0007,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2PartCutLog.GetValue(), 0.0008,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3PartCutLog.GetValue(), 0.0009,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4PartCutLog.GetValue(), 0.00011,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5PartCutLog.GetValue(), 0.00012,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1GapCutLog.GetValue(), 0.00013,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2GapCutLog.GetValue(), 0.00014,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3GapCutLog.GetValue(), 0.00015,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4GapCutLog.GetValue(), 0.00016,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5GapCutLog.GetValue(), 0.00017,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1GapCutLog.GetValue(), 0.00018,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2GapCutLog.GetValue(), 0.00019,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3GapCutLog.GetValue(), 0.00021,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4GapCutLog.GetValue(), 0.00022,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5GapCutLog.GetValue(), 0.00023,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass1ClearCutLog.GetValue(), 0.00024,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass2ClearCutLog.GetValue(), 0.00025,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass3ClearCutLog.GetValue(), 0.00026,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass4ClearCutLog.GetValue(), 0.00027,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3SmallClass5ClearCutLog.GetValue(), 0.00028,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass1ClearCutLog.GetValue(), 0.00029,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass2ClearCutLog.GetValue(), 0.00031,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass3ClearCutLog.GetValue(), 0.00032,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass4ClearCutLog.GetValue(), 0.00033,
					0.0001);
			assertEquals(oSubs.m_fSpGroup3LargeClass5ClearCutLog.GetValue(), 0.00034,
					0.0001);
			assertEquals(oSubs.m_fGapCutScarSoil.GetValue(), 0.035, 0.0001);
			assertEquals(oSubs.m_fGapCutTipup.GetValue(), 0.036, 0.0001);
			assertEquals(oSubs.m_fPartialCutScarSoil.GetValue(), 0.037, 0.0001);
			assertEquals(oSubs.m_fPartialCutTipup.GetValue(), 0.038, 0.0001);
			assertEquals(oSubs.m_fInitCondScarSoil.GetValue(), 0.039, 0.0001);
			assertEquals(oSubs.m_fInitCondTipup.GetValue(), 0.041, 0.0001);
			assertEquals(oSubs.m_fClearCutScarSoil.GetValue(), 0.042, 0.0001);
			assertEquals(oSubs.m_fClearCutTipup.GetValue(), 0.043, 0.0001);
			assertEquals(oSubs.m_fScarSoilA.GetValue(), -0.044, 0.0001);
			assertEquals(oSubs.m_fScarSoilB.GetValue(), 0.045, 0.0001);
			assertEquals(oSubs.m_fTipUpA.GetValue(), -0.046, 0.0001);
			assertEquals(oSubs.m_fTipUpB.GetValue(), 0.047, 0.0001);
			assertEquals(oSubs.m_iMaxDecayTime.GetValue(), 2);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(0)).floatValue(), 0.048, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(1)).floatValue(), 0.049, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue().elementAt(2)).floatValue(), 0.053, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(0)).floatValue(), 0.051, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(1)).floatValue(), 0.052, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue().elementAt(2)).floatValue(), 0.054, 0.0001);
			assertEquals(oSubs.m_fRootTipupFactor.GetValue(), 3, 0.0001);
			assertEquals(oSubs.m_fMossProportion.GetValue(), 0.4, 0.0001);
			assertEquals(oSubs.m_iDirectionalTreeFall.GetValue(), 0);
			assertEquals(oSubs.m_fInitSmallLogMeanDiam.GetValue(), 20, 0.0001);
			assertEquals(oSubs.m_fInitLargeLogMeanDiam.GetValue(), 50, 0.0001);
			assertEquals(oSubs.m_fPartCutSmallLogMeanDiam.GetValue(), 21, 0.0001);
			assertEquals(oSubs.m_fPartCutLargeLogMeanDiam.GetValue(), 51, 0.0001);
			assertEquals(oSubs.m_fGapCutSmallLogMeanDiam.GetValue(), 22, 0.0001);
			assertEquals(oSubs.m_fGapCutLargeLogMeanDiam.GetValue(), 52, 0.0001);
			assertEquals(oSubs.m_fClearCutSmallLogMeanDiam.GetValue(), 23, 0.0001);
			assertEquals(oSubs.m_fClearCutLargeLogMeanDiam.GetValue(), 53, 0.0001);
		} catch (ModelException oErr) {
			fail("Substrate parameter file read test failed with message "
					+ oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			if (sFileName != null) {
				new File(sFileName).delete();
			}
		}
	}

	/**
	 * Tests ValidateData().
	 * 
	 * @throws ModelException
	 *           if there's an uncaught validation problem.
	 */
	public void testValidateData() throws ModelException {
		GUIManager oManager = null;
		String sFileName = null;
		SubstrateBehaviors oSubs = null;
		try {

			oManager = new GUIManager(null);

			// Valid file 1
			oManager.ClearCurrentData();
			sFileName = WriteXMLValidFile1();
			oManager.InputXMLParameterFile(sFileName);
			oSubs = oManager.GetSubstrateBehaviors();
			oSubs.ValidateData(oManager.GetTreePopulation());
		} catch (ModelException oErr) {
			fail("Substrate validation failed with message " + oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}

		// Case: root tip-up factor is negative
		try {
			oSubs.m_fRootTipupFactor.SetValue(-3);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch negative root tipup factor.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fRootTipupFactor.SetValue(3);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: max decay time is negative
		try {
			oSubs.m_iMaxDecayTime.SetValue(-3);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch negative max decay time.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_iMaxDecayTime.SetValue(3);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: proportion of dead that fall is not a proportion
		try {
			oSubs.mp_fProportionOfDeadThatFall.GetValue().clear();
			oSubs.mp_fProportionOfDeadThatFall.GetValue().add(new Float(0.2));
			oSubs.mp_fProportionOfDeadThatFall.GetValue().add(new Float(1.2));
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch bad value for proportion of dead that fall.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		try {
			sFileName = WriteXMLValidFile1();
			oManager.InputXMLParameterFile(sFileName);
			oSubs = oManager.GetSubstrateBehaviors();
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}

		// Case: proportion of fallen that uproot is not a proportion
		try {
			oSubs.mp_fProportionOfFallenThatUproot.GetValue().clear();
			oSubs.mp_fProportionOfFallenThatUproot.GetValue().add(new Float(0.2));
			oSubs.mp_fProportionOfFallenThatUproot.GetValue().add(new Float(-1.2));
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch bad value for proportion of fallen that uproot.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		try {
			sFileName = WriteXMLValidFile1();
			oManager.InputXMLParameterFile(sFileName);
			oSubs = oManager.GetSubstrateBehaviors();
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}

		// Case: proportion of snags that uproot is not a proportion
		try {
			oSubs.mp_fProportionOfSnagsThatUproot.GetValue().clear();
			oSubs.mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(0.2));
			oSubs.mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(-1.2));
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch bad value for proportion of snags that uproot.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		try {
			sFileName = WriteXMLValidFile1();
			oManager.InputXMLParameterFile(sFileName);
			oSubs = oManager.GetSubstrateBehaviors();
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}

		// Case: proportion of moss isn't a proportion
		try {
			oSubs.m_fMossProportion.SetValue(-3);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch bad moss proportion value.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fMossProportion.SetValue((float) 0.3);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: initial conditions add up to greater than 1
		try {
			oSubs.m_fInitCondDecLog.SetValue((float) 0.9);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch initial conditions greater than 1.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fInitCondDecLog.SetValue((float) 0.01);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: partial cut harvest conditions add up to greater than 1
		try {
			oSubs.m_fPartialCutDecLog.SetValue((float) 0.9);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch partial cut conditions greater than 1.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fPartialCutDecLog.SetValue((float) 0.01);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: gap cut harvest conditions add up to greater than 1
		try {
			oSubs.m_fGapCutDecLog.SetValue((float) 0.9);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch gap cut conditions greater than 1.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fGapCutDecLog.SetValue((float) 0.01);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// Case: clear cut harvest conditions add up to greater than 1
		try {
			oSubs.m_fClearCutDecLog.SetValue((float) 0.9);
			oSubs.ValidateData(oManager.GetTreePopulation());
			fail("Substrate didn't catch clear cut conditions greater than 1.");
		} catch (ModelException oErr) {
			;
		}
		// Reset
		oSubs.m_fClearCutDecLog.SetValue((float) 0.01);
		oSubs.ValidateData(oManager.GetTreePopulation());

		// *******************************
		// Detailed substrate testing
		// *******************************
		try {
			try {

				oManager = new GUIManager(null);

				// Valid file 1
				sFileName = WriteXMLValidFile2();
				oManager.InputXMLParameterFile(sFileName);
				oSubs = oManager.GetSubstrateBehaviors();
				oSubs.ValidateData(oManager.GetTreePopulation());
			} catch (ModelException oErr) {
				fail("Substrate validation failed with message " + oErr.getMessage());
			} catch (java.io.IOException oE) {
				fail("Caught IOException.  Message: " + oE.getMessage());
			} finally {
				new File(sFileName).delete();
			}

			// Case: betas too large
			float fBadValue = 100000, fTemp = 0;
			try {
				fTemp = oSubs.m_fScarSoilB.GetValue();
				oSubs.m_fScarSoilB.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fScarSoilB.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fTipUpB.GetValue();
				oSubs.m_fTipUpB.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fTipUpB.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass1Beta.GetValue();
				oSubs.m_fSpGroup1SmallClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass2Beta.GetValue();
				oSubs.m_fSpGroup1SmallClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass3Beta.GetValue();
				oSubs.m_fSpGroup1SmallClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass4Beta.GetValue();
				oSubs.m_fSpGroup1SmallClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass5Beta.GetValue();
				oSubs.m_fSpGroup1SmallClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass1Beta.GetValue();
				oSubs.m_fSpGroup2SmallClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass2Beta.GetValue();
				oSubs.m_fSpGroup2SmallClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass3Beta.GetValue();
				oSubs.m_fSpGroup2SmallClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass4Beta.GetValue();
				oSubs.m_fSpGroup2SmallClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass5Beta.GetValue();
				oSubs.m_fSpGroup2SmallClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass1Beta.GetValue();
				oSubs.m_fSpGroup3SmallClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass2Beta.GetValue();
				oSubs.m_fSpGroup3SmallClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass3Beta.GetValue();
				oSubs.m_fSpGroup3SmallClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass4Beta.GetValue();
				oSubs.m_fSpGroup3SmallClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass5Beta.GetValue();
				oSubs.m_fSpGroup3SmallClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1Beta.GetValue();
				oSubs.m_fSpGroup1LargeClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass2Beta.GetValue();
				oSubs.m_fSpGroup1LargeClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass3Beta.GetValue();
				oSubs.m_fSpGroup1LargeClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass4Beta.GetValue();
				oSubs.m_fSpGroup1LargeClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass5Beta.GetValue();
				oSubs.m_fSpGroup1LargeClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass1Beta.GetValue();
				oSubs.m_fSpGroup2LargeClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass2Beta.GetValue();
				oSubs.m_fSpGroup2LargeClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass3Beta.GetValue();
				oSubs.m_fSpGroup2LargeClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass4Beta.GetValue();
				oSubs.m_fSpGroup2LargeClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass5Beta.GetValue();
				oSubs.m_fSpGroup2LargeClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass1Beta.GetValue();
				oSubs.m_fSpGroup3LargeClass1Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass1Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass2Beta.GetValue();
				oSubs.m_fSpGroup3LargeClass2Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass2Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass3Beta.GetValue();
				oSubs.m_fSpGroup3LargeClass3Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass3Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass4Beta.GetValue();
				oSubs.m_fSpGroup3LargeClass4Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass4Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass5Beta.GetValue();
				oSubs.m_fSpGroup3LargeClass5Beta.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge beta.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass5Beta.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			// Case: positive alphas
			fBadValue = 1;
			try {
				fTemp = oSubs.m_fScarSoilA.GetValue();
				oSubs.m_fScarSoilA.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fScarSoilA.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fTipUpA.GetValue();
				oSubs.m_fTipUpA.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fTipUpA.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass1Alpha.GetValue();
				oSubs.m_fSpGroup1SmallClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass2Alpha.GetValue();
				oSubs.m_fSpGroup1SmallClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass3Alpha.GetValue();
				oSubs.m_fSpGroup1SmallClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass4Alpha.GetValue();
				oSubs.m_fSpGroup1SmallClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1SmallClass5Alpha.GetValue();
				oSubs.m_fSpGroup1SmallClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1SmallClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass1Alpha.GetValue();
				oSubs.m_fSpGroup2SmallClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass2Alpha.GetValue();
				oSubs.m_fSpGroup2SmallClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass3Alpha.GetValue();
				oSubs.m_fSpGroup2SmallClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass4Alpha.GetValue();
				oSubs.m_fSpGroup2SmallClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2SmallClass5Alpha.GetValue();
				oSubs.m_fSpGroup2SmallClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2SmallClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass1Alpha.GetValue();
				oSubs.m_fSpGroup3SmallClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass2Alpha.GetValue();
				oSubs.m_fSpGroup3SmallClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass3Alpha.GetValue();
				oSubs.m_fSpGroup3SmallClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass4Alpha.GetValue();
				oSubs.m_fSpGroup3SmallClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3SmallClass5Alpha.GetValue();
				oSubs.m_fSpGroup3SmallClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3SmallClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1Alpha.GetValue();
				oSubs.m_fSpGroup1LargeClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass2Alpha.GetValue();
				oSubs.m_fSpGroup1LargeClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass3Alpha.GetValue();
				oSubs.m_fSpGroup1LargeClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass4Alpha.GetValue();
				oSubs.m_fSpGroup1LargeClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass5Alpha.GetValue();
				oSubs.m_fSpGroup1LargeClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass1Alpha.GetValue();
				oSubs.m_fSpGroup2LargeClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass2Alpha.GetValue();
				oSubs.m_fSpGroup2LargeClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass3Alpha.GetValue();
				oSubs.m_fSpGroup2LargeClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass4Alpha.GetValue();
				oSubs.m_fSpGroup2LargeClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup2LargeClass5Alpha.GetValue();
				oSubs.m_fSpGroup2LargeClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup2LargeClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass1Alpha.GetValue();
				oSubs.m_fSpGroup3LargeClass1Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass1Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass2Alpha.GetValue();
				oSubs.m_fSpGroup3LargeClass2Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass2Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass3Alpha.GetValue();
				oSubs.m_fSpGroup3LargeClass3Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass3Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass4Alpha.GetValue();
				oSubs.m_fSpGroup3LargeClass4Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass4Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup3LargeClass5Alpha.GetValue();
				oSubs.m_fSpGroup3LargeClass5Alpha.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch huge alpha.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup3LargeClass5Alpha.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			// Case: initial conditions proportions are greater than one
			fBadValue = 1;
			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1InitLog.GetValue();
				oSubs.m_fSpGroup1LargeClass1InitLog.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad initial conditions.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1InitLog.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1PartCutLog.GetValue();
				oSubs.m_fSpGroup1LargeClass1PartCutLog.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad initial conditions.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1PartCutLog.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1GapCutLog.GetValue();
				oSubs.m_fSpGroup1LargeClass1GapCutLog.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad initial conditions.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1GapCutLog.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fSpGroup1LargeClass1ClearCutLog.GetValue();
				oSubs.m_fSpGroup1LargeClass1ClearCutLog.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad initial conditions.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fSpGroup1LargeClass1ClearCutLog.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			// Case: log mean diameters are not on the correct side of the log
			// boundary
			fBadValue = oSubs.m_fLogSizeClassBoundary.GetValue() + 1;
			try {
				fTemp = oSubs.m_fInitSmallLogMeanDiam.GetValue();
				oSubs.m_fInitSmallLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad small log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fInitSmallLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fPartCutSmallLogMeanDiam.GetValue();
				oSubs.m_fPartCutSmallLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad small log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fPartCutSmallLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fGapCutSmallLogMeanDiam.GetValue();
				oSubs.m_fGapCutSmallLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad small log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fGapCutSmallLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fClearCutSmallLogMeanDiam.GetValue();
				oSubs.m_fClearCutSmallLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad small log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fClearCutSmallLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			fBadValue = oSubs.m_fLogSizeClassBoundary.GetValue() - 1;
			try {
				fTemp = oSubs.m_fInitLargeLogMeanDiam.GetValue();
				oSubs.m_fInitLargeLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad large log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fInitLargeLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fPartCutLargeLogMeanDiam.GetValue();
				oSubs.m_fPartCutLargeLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad large log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fPartCutLargeLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fGapCutLargeLogMeanDiam.GetValue();
				oSubs.m_fGapCutLargeLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad large log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fGapCutLargeLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			try {
				fTemp = oSubs.m_fClearCutLargeLogMeanDiam.GetValue();
				oSubs.m_fClearCutLargeLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch bad large log mean diameter.");
			} catch (ModelException oErr) {
				;
			}
			oSubs.m_fClearCutLargeLogMeanDiam.SetValue(fTemp);
			oSubs.ValidateData(oManager.GetTreePopulation());

			// Case - log diams ignored if harvest not used
			oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Harvest").m_bIsEnabled = false;
			try {
				fTemp = oSubs.m_fClearCutLargeLogMeanDiam.GetValue();
				oSubs.m_fClearCutLargeLogMeanDiam.SetValue(fBadValue);
				oSubs.ValidateData(oManager.GetTreePopulation());
			} catch (ModelException oErr) {
				fail("Substrate validation failed with message " + oErr.getMessage());
			}
			
      //Case - error if snag dynamics not used
			try {
				oManager.GetSnagDynamicsBehaviors()
    		.GetBehaviorByKey("Snag Decay Class Dynamics").m_bIsEnabled = false;
				oSubs.ValidateData(oManager.GetTreePopulation());
				fail("Substrate didn't catch missing snag dynamics behavior.");
			} catch (ModelException oErr) {;}
		} catch (ModelException oErr) {
			fail("Substrate validation failed with message " + oErr.getMessage());
		} finally {
			new File(sFileName).delete();
		}

		System.out.println("Substrate ValidateData testing succeeded.");
	}

	/**
	 * Tests species changes. Even though SubstrateBehaviors doesn't explicitly
	 * have code for changing species, this makes sure that it is treated
	 * correctly.
	 */
	public void testChangeOfSpecies() {
		GUIManager oManager = null;
		String sFileName = null;
		try {
			oManager = new GUIManager(null);
			oManager.ClearCurrentData();
			sFileName = WriteXMLValidFile2();
			oManager.InputXMLParameterFile(sFileName);
			SubstrateBehaviors oSubs = oManager.GetSubstrateBehaviors();

			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.048, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.049, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(2)).floatValue(), 0.053, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(0)).floatValue(), 0.051, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.052, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(2)).floatValue(), 0.054, 0.0001);

			ModelEnum oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(0);
			assertEquals(oEnum.GetValue(), 1);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(1);
			assertEquals(oEnum.GetValue(), 2);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(2);
			assertEquals(oEnum.GetValue(), 3);

			// Now change the species
			String[] sNewSpecies = new String[] { "Species 4", "Species 3", 
					"Species 2",	"Species 1" };
			oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
			oSubs = oManager.GetSubstrateBehaviors();

			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(3)).floatValue(), 0.048, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(2)).floatValue(), 0.049, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfFallenThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.053, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(3)).floatValue(), 0.051, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(2)).floatValue(), 0.052, 0.0001);
			assertEquals(((Float) oSubs.mp_fProportionOfSnagsThatUproot.GetValue()
					.elementAt(1)).floatValue(), 0.054, 0.0001);

			int i;
			assertEquals(4, oSubs.mp_iSpeciesGroup.GetValue().size());
			for (i = 0; i < 4; i++) {
				assertTrue(null != oSubs.mp_iSpeciesGroup.GetValue().elementAt(i));
			}
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(3);
			assertEquals(oEnum.GetValue(), 1);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(2);
			assertEquals(oEnum.GetValue(), 2);
			oEnum = (ModelEnum) oSubs.mp_iSpeciesGroup.GetValue().elementAt(1);
			assertEquals(oEnum.GetValue(), 3);

			System.out.println("Change of species test succeeded.");
		} catch (ModelException oErr) {
			fail("Change of species (substrate) test failed with message "
					+ oErr.getMessage());
		} catch (java.io.IOException oE) {
			fail("Caught IOException.  Message: " + oE.getMessage());
		} finally {
			new File(sFileName).delete();
		}
	}

	/**
	 * Writes an old-school substrate file to test backwards compatibility.
	 * 
	 * @return The file name.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * 
	 * <br>
	 * Edit history: <br>
	 * ------------------ <br>
	 * June 3, 2005: Created (LEM)
	 */
	protected String WriteXMLOldFile1() throws java.io.IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		WritePlotAndTrees(oOut);
		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>substrate</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");

		oOut.write("<substrate>");
		oOut.write("<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>");
		oOut.write("<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>");
		oOut.write("<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>");
		oOut.write("<su_organicDecayBeta>4.3</su_organicDecayBeta>");
		oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
		oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
		oOut.write("<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>");
		oOut.write("<su_maxNumberDecayYears>50</su_maxNumberDecayYears>");
		oOut.write("<su_initialGapMoss>0.18</su_initialGapMoss>");
		oOut.write("<su_initialOrganic>0.3</su_initialOrganic>");
		oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
		oOut.write("<su_initialMossyLog>0.11</su_initialMossyLog>");
		oOut.write("<su_partialCutGapMoss>0.17</su_partialCutGapMoss>");
		oOut.write("<su_partialCutOrganic>0.15</su_partialCutOrganic>");
		oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
		oOut.write("<su_partialCutMossyLog>0.04</su_partialCutMossyLog>");
		oOut.write("<su_gapCutGapMoss>0.45</su_gapCutGapMoss>");
		oOut.write("<su_gapCutOrganic>0.34</su_gapCutOrganic>");
		oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
		oOut.write("<su_gapCutMossyLog>0.12</su_gapCutMossyLog>");
		oOut.write("<su_clearCutGapMoss>0.36</su_clearCutGapMoss>");
		oOut.write("<su_clearCutOrganic>0.35</su_clearCutOrganic>");
		oOut.write("<su_clearCutFreshLog>0.16</su_clearCutFreshLog>");
		oOut.write("<su_clearCutMossyLog>0.01</su_clearCutMossyLog>");
		oOut.write("<su_propOfDeadFall>");
		oOut.write("<su_podfVal species=\"Species_1\">0.7</su_podfVal>");
		oOut.write("<su_podfVal species=\"Species_2\">0.6</su_podfVal>");
		oOut.write("</su_propOfDeadFall>");
		oOut.write("<su_propOfFallUproot>");
		oOut.write("<su_pofuVal species=\"Species_1\">0.5</su_pofuVal>");
		oOut.write("<su_pofuVal species=\"Species_2\">0.4</su_pofuVal>");
		oOut.write("</su_propOfFallUproot>");
		oOut.write("<su_propOfSnagsUproot>");
		oOut.write("<su_posuVal species=\"Species_1\">0.3</su_posuVal>");
		oOut.write("<su_posuVal species=\"Species_2\">0.2</su_posuVal>");
		oOut.write("</su_propOfSnagsUproot>");
		oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
		oOut.write("</substrate>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}

	/**
	 * Writes a 6.05 substrate file.
	 * 
	 * @throws IOException
	 *           If there is a problem writing the file.
	 * @return String Filename of file written.
	 */
	protected String WriteXMLOldFile2() throws java.io.IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		WritePlotAndTrees(oOut);

		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>substrate</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");

		oOut.write("<substrate>");
		oOut.write("<su_scarSoilDecayAlpha>-5.1</su_scarSoilDecayAlpha>");
		oOut.write("<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>");
		oOut.write("<su_tipupDecayAlpha>-7.0</su_tipupDecayAlpha>");
		oOut.write("<su_tipupDecayBeta>4.3</su_tipupDecayBeta>");
		oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
		oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
		oOut.write("<su_DecayedLogToForestFloor>0.75</su_DecayedLogToForestFloor>");
		oOut.write("<su_maxNumberDecayYears>3</su_maxNumberDecayYears>");
		oOut.write("<su_initialScarSoil>0.23</su_initialScarSoil>");
		oOut.write("<su_initialTipup>0.2</su_initialTipup>");
		oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
		oOut.write("<su_initialDecayedLog>0.11</su_initialDecayedLog>");
		oOut.write("<su_partialCutScarSoil>0.17</su_partialCutScarSoil>");
		oOut.write("<su_partialCutTipup>0.15</su_partialCutTipup>");
		oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
		oOut.write("<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>");
		oOut.write("<su_gapCutScarSoil>0.45</su_gapCutScarSoil>");
		oOut.write("<su_gapCutTipup>0.34</su_gapCutTipup>");
		oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
		oOut.write("<su_gapCutDecayedLog>0.12</su_gapCutDecayedLog>");
		oOut.write("<su_clearCutScarSoil>0.36</su_clearCutScarSoil>");
		oOut.write("<su_clearCutTipup>0.32</su_clearCutTipup>");
		oOut.write("<su_clearCutFreshLog>0.13</su_clearCutFreshLog>");
		oOut.write("<su_clearCutDecayedLog>0.02</su_clearCutDecayedLog>");
		oOut.write("<su_propOfDeadFall>");
		oOut.write("<su_podfVal species=\"Species_1\">1</su_podfVal>");
		oOut.write("<su_podfVal species=\"Species_2\">0.2</su_podfVal>");
		oOut.write("</su_propOfDeadFall>");
		oOut.write("<su_propOfFallUproot>");
		oOut.write("<su_pofuVal species=\"Species_1\">0</su_pofuVal>");
		oOut.write("<su_pofuVal species=\"Species_2\">0.18</su_pofuVal>");
		oOut.write("</su_propOfFallUproot>");
		oOut.write("<su_propOfSnagsUproot>");
		oOut.write("<su_posuVal species=\"Species_1\">0.4</su_posuVal>");
		oOut.write("<su_posuVal species=\"Species_2\">0.8</su_posuVal>");
		oOut.write("</su_propOfSnagsUproot>");
		oOut.write("<su_mossProportion>0.7</su_mossProportion>");
		oOut.write("<su_directionalTreeFall>1</su_directionalTreeFall>");
		oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
		oOut.write("</substrate>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}

	/**
	 * Writes a 6.05 substrate file.
	 * 
	 * @throws IOException
	 *           If there is a problem writing the file.
	 * @return String Filename of file written.
	 */
	protected String WriteXMLValidFile1() throws java.io.IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		WritePlotAndTrees(oOut);

		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>substrate</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");

		oOut.write("<substrate>");
		oOut.write("<su_scarSoilDecayAlpha>-5.1</su_scarSoilDecayAlpha>");
		oOut.write("<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>");
		oOut.write("<su_tipupDecayAlpha>-7.0</su_tipupDecayAlpha>");
		oOut.write("<su_tipupDecayBeta>4.3</su_tipupDecayBeta>");
		oOut.write("<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>");
		oOut.write("<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>");
		oOut.write("<su_decayedLogDecayAlpha>-0.2</su_decayedLogDecayAlpha>");
		oOut.write("<su_decayedLogDecayBeta>5.2</su_decayedLogDecayBeta>");
		oOut.write("<su_maxNumberDecayYears>3</su_maxNumberDecayYears>");
		oOut.write("<su_initialScarSoil>0.23</su_initialScarSoil>");
		oOut.write("<su_initialTipup>0.2</su_initialTipup>");
		oOut.write("<su_initialFreshLog>0.01</su_initialFreshLog>");
		oOut.write("<su_initialDecayedLog>0.11</su_initialDecayedLog>");
		oOut.write("<su_partialCutScarSoil>0.17</su_partialCutScarSoil>");
		oOut.write("<su_partialCutTipup>0.15</su_partialCutTipup>");
		oOut.write("<su_partialCutFreshLog>0.06</su_partialCutFreshLog>");
		oOut.write("<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>");
		oOut.write("<su_gapCutScarSoil>0.45</su_gapCutScarSoil>");
		oOut.write("<su_gapCutTipup>0.34</su_gapCutTipup>");
		oOut.write("<su_gapCutFreshLog>0.09</su_gapCutFreshLog>");
		oOut.write("<su_gapCutDecayedLog>0.12</su_gapCutDecayedLog>");
		oOut.write("<su_clearCutScarSoil>0.36</su_clearCutScarSoil>");
		oOut.write("<su_clearCutTipup>0.32</su_clearCutTipup>");
		oOut.write("<su_clearCutFreshLog>0.13</su_clearCutFreshLog>");
		oOut.write("<su_clearCutDecayedLog>0.02</su_clearCutDecayedLog>");
		oOut.write("<su_propOfDeadFall>");
		oOut.write("<su_podfVal species=\"Species_1\">1</su_podfVal>");
		oOut.write("<su_podfVal species=\"Species_2\">0.2</su_podfVal>");
		oOut.write("</su_propOfDeadFall>");
		oOut.write("<su_propOfFallUproot>");
		oOut.write("<su_pofuVal species=\"Species_1\">0</su_pofuVal>");
		oOut.write("<su_pofuVal species=\"Species_2\">0.18</su_pofuVal>");
		oOut.write("</su_propOfFallUproot>");
		oOut.write("<su_propOfSnagsUproot>");
		oOut.write("<su_posuVal species=\"Species_1\">0.4</su_posuVal>");
		oOut.write("<su_posuVal species=\"Species_2\">0.8</su_posuVal>");
		oOut.write("</su_propOfSnagsUproot>");
		oOut.write("<su_mossProportion>0.7</su_mossProportion>");
		oOut.write("<su_directionalTreeFall>1</su_directionalTreeFall>");
		oOut.write("<su_rootTipupFactor>3.1</su_rootTipupFactor>");
		oOut.write("</substrate>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}

	/**
	 * This writes plot and tree information.
	 * 
	 * @param oOut
	 *          FileWriter
	 * @throws IOException
	 */
	private void WritePlotAndTrees(FileWriter oOut) throws java.io.IOException {
		oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
		oOut.write("<paramFile fileCode=\"06010101\">");
		oOut.write("<plot>");
		oOut.write("<timesteps>1</timesteps>");
		oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
		oOut.write("<randomSeed>1</randomSeed>");
		oOut.write("<plot_lenX>200.0</plot_lenX>");
		oOut.write("<plot_lenY>200.0</plot_lenY>");
		oOut.write("<plot_latitude>55.37</plot_latitude>");
		oOut.write("</plot>");
		oOut.write("<trees>");
		oOut.write("<tr_speciesList>");
		oOut.write("<tr_species speciesName=\"Species_1\" />");
		oOut.write("<tr_species speciesName=\"Species_2\" />");
		oOut.write("<tr_species speciesName=\"Species_3\" />");
		oOut.write("</tr_speciesList>");
		oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
		oOut.write("<tr_minAdultDBH>");
		oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
		oOut.write("</tr_minAdultDBH>");
		oOut.write("</trees>");
		oOut.write("<allometry>");
		oOut.write("<tr_canopyHeight>");
		oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_3\">39.48</tr_chVal>");
		oOut.write("</tr_canopyHeight>");
		oOut.write("<tr_stdAsympCrownRad>");
		oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
		oOut.write("</tr_stdAsympCrownRad>");
		oOut.write("<tr_stdCrownRadExp>");
		oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
		oOut.write("</tr_stdCrownRadExp>");
		oOut.write("<tr_conversionDiam10ToDBH>");
		oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
		oOut.write("</tr_conversionDiam10ToDBH>");
		oOut.write("<tr_stdAsympCrownHt>");
		oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
		oOut.write("</tr_stdAsympCrownHt>");
		oOut.write("<tr_stdCrownHtExp>");
		oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
		oOut.write("</tr_stdCrownHtExp>");
		oOut.write("<tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
		oOut.write("</tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_slopeOfAsymHeight>");
		oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>");
		oOut.write("</tr_slopeOfAsymHeight>");
		oOut.write("</allometry>");
	}

	/**
	 * Writes a file with no analysis behaviors.
	 * 
	 * @return String Filename written.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 */
	protected String WriteNeutralFile() throws java.io.IOException {
		String sFileName = "\\loratestxml1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
		oOut.write("<paramFile fileCode=\"06010101\">");
		oOut.write("<plot>");
		oOut.write("<timesteps>2</timesteps>");
		oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
		oOut.write("<randomSeed>1</randomSeed>");
		oOut.write("<plot_lenX>200.0</plot_lenX>");
		oOut.write("<plot_lenY>200.0</plot_lenY>");
		oOut.write("<plot_latitude>55.37</plot_latitude>");
		oOut.write("</plot>");
		oOut.write("<trees>");
		oOut.write("<tr_speciesList>");
		oOut.write("<tr_species speciesName=\"Species_1\" />");
		oOut.write("<tr_species speciesName=\"Species_2\" />");
		oOut.write("<tr_species speciesName=\"Species_3\" />");
		oOut.write("</tr_speciesList>");
		oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
		oOut.write("<tr_canopyHeight>");
		oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
		oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
		oOut.write("</tr_canopyHeight>");
		oOut.write("<tr_stdAsympCrownRad>");
		oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
		oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
		oOut.write("</tr_stdAsympCrownRad>");
		oOut.write("<tr_stdCrownRadExp>");
		oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
		oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
		oOut.write("</tr_stdCrownRadExp>");
		oOut.write("<tr_conversionDiam10ToDBH>");
		oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
		oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
		oOut.write("</tr_conversionDiam10ToDBH>");
		oOut.write("<tr_stdAsympCrownHt>");
		oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
		oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
		oOut.write("</tr_stdAsympCrownHt>");
		oOut.write("<tr_stdCrownHtExp>");
		oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
		oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
		oOut.write("</tr_stdCrownHtExp>");
		oOut.write("<tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
		oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
		oOut.write("</tr_slopeOfHeight-Diam10>");
		oOut.write("<tr_minAdultDBH>");
		oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
		oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
		oOut.write("</tr_minAdultDBH>");
		oOut.write("<tr_slopeOfAsymHeight>");
		oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
		oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
		oOut.write("</tr_slopeOfAsymHeight>");
		oOut.write("</trees>");
		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>radiallight</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");
		oOut.write("</paramFile>");

		oOut.close();
		return sFileName;
	}

	/**
	 * Writes a detailed substrate file.
	 * 
	 * @throws IOException
	 *           If there is a problem writing the file.
	 * @return String Filename of file written.
	 */
	protected String WriteXMLValidFile2() throws IOException {
		String sFileName = "\\testfile1.xml";
		FileWriter oOut = new FileWriter(sFileName);

		WritePlotAndTrees(oOut);

		oOut.write("<behaviorList>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>harvest</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("</behavior>");
		// Have to have a mortality behavior so dead is registered
		oOut.write("<behavior>");
		oOut.write("<behaviorName>adultstochasticmort</behaviorName>");
		oOut.write("<version>1</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("</behavior>");
		// Have to have a snag dynamics behavior to register
		// appropriate data members
		oOut.write("<behavior>");
		oOut.write("<behaviorName>snag decay class dynamics</behaviorName>");
		oOut.write("<version>1.0</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("</behavior>");
		oOut.write("<behavior>");
		oOut.write("<behaviorName>detailedsubstrate</behaviorName>");
		oOut.write("<version>1.0</version>");
		oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
		oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
		oOut.write("</behavior>");
		oOut.write("</behaviorList>");
		oOut.write("<harvest>");
		oOut.write("<ha_cutEvent>");
		oOut.write("<ha_applyToSpecies species=\"Species_1\" />");
		oOut.write("<ha_timestep>2</ha_timestep>");
		oOut.write("<ha_cutType>gap</ha_cutType>");
		oOut.write("<ha_cutAmountType>percent of density</ha_cutAmountType>");
		oOut.write("<ha_dbhRange>");
		oOut.write("<ha_low>0.0</ha_low>");
		oOut.write("<ha_high>25.0</ha_high>");
		oOut.write("<ha_amountToCut>100.0</ha_amountToCut>");
		oOut.write("</ha_dbhRange>");
		oOut.write("<ha_applyToCell x=\"0\" y=\"2\" />");
		oOut.write("</ha_cutEvent>");
		oOut.write("</harvest>");
		oOut.write("<mortality>");
		oOut.write("<mo_nonSizeDepMort>");
		oOut.write("<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>");
		oOut.write("<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>");
		oOut.write("<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>");
		oOut.write("</mo_nonSizeDepMort>");
		oOut.write("</mortality>");
		oOut.write("<substrate>");
		oOut.write("<su_logSizeClassBoundary>40</su_logSizeClassBoundary>");
		oOut.write("<su_logSpGroupAssignment>");
		oOut.write("<su_lsgaVal species=\"Species_1\">1</su_lsgaVal>");
		oOut.write("<su_lsgaVal species=\"Species_2\">2</su_lsgaVal>");
		oOut.write("<su_lsgaVal species=\"Species_3\">3</su_lsgaVal>");
		oOut.write("</su_logSpGroupAssignment>");
		oOut.write("<su_propFallenTreesLogDecayClass1>0.1</su_propFallenTreesLogDecayClass1>");
		oOut.write("<su_propFallenTreesLogDecayClass2>0.2</su_propFallenTreesLogDecayClass2>");
		oOut.write("<su_propFallenTreesLogDecayClass3>0.3</su_propFallenTreesLogDecayClass3>");
		oOut.write("<su_propFallenTreesLogDecayClass4>0.04</su_propFallenTreesLogDecayClass4>");
		oOut.write("<su_propFallenTreesLogDecayClass5>0.36</su_propFallenTreesLogDecayClass5>");
		oOut.write("<su_propFallenSnagsLogDecayClass1>0.006</su_propFallenSnagsLogDecayClass1>");
		oOut.write("<su_propFallenSnagsLogDecayClass2>0.007</su_propFallenSnagsLogDecayClass2>");
		oOut.write("<su_propFallenSnagsLogDecayClass3>0.008</su_propFallenSnagsLogDecayClass3>");
		oOut.write("<su_propFallenSnagsLogDecayClass4>0.009</su_propFallenSnagsLogDecayClass4>");
		oOut.write("<su_propFallenSnagsLogDecayClass5>0.97</su_propFallenSnagsLogDecayClass5>");
		oOut.write("<su_logSpGroup1SmallDecayClass1DecayAlpha>-1</su_logSpGroup1SmallDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup1SmallDecayClass2DecayAlpha>-0.8</su_logSpGroup1SmallDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup1SmallDecayClass3DecayAlpha>-1.2</su_logSpGroup1SmallDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup1SmallDecayClass4DecayAlpha>-0.5</su_logSpGroup1SmallDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup1SmallDecayClass5DecayAlpha>-2</su_logSpGroup1SmallDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup1LargeDecayClass1DecayAlpha>-3</su_logSpGroup1LargeDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup1LargeDecayClass2DecayAlpha>-4</su_logSpGroup1LargeDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup1LargeDecayClass3DecayAlpha>-5</su_logSpGroup1LargeDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup1LargeDecayClass4DecayAlpha>-6</su_logSpGroup1LargeDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup1LargeDecayClass5DecayAlpha>-7</su_logSpGroup1LargeDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup1SmallDecayClass1DecayBeta>-8</su_logSpGroup1SmallDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup1SmallDecayClass2DecayBeta>-9</su_logSpGroup1SmallDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup1SmallDecayClass3DecayBeta>-10</su_logSpGroup1SmallDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup1SmallDecayClass4DecayBeta>-11</su_logSpGroup1SmallDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup1SmallDecayClass5DecayBeta>-12</su_logSpGroup1SmallDecayClass5DecayBeta>");
		oOut.write("<su_logSpGroup1LargeDecayClass1DecayBeta>-13</su_logSpGroup1LargeDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup1LargeDecayClass2DecayBeta>-14</su_logSpGroup1LargeDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup1LargeDecayClass3DecayBeta>-15</su_logSpGroup1LargeDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup1LargeDecayClass4DecayBeta>-16</su_logSpGroup1LargeDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup1LargeDecayClass5DecayBeta>-17</su_logSpGroup1LargeDecayClass5DecayBeta>");
		oOut.write("<su_initialLogSpGroup1SmallDecayClass1>0.0011</su_initialLogSpGroup1SmallDecayClass1>");
		oOut.write("<su_initialLogSpGroup1SmallDecayClass2>0.0012</su_initialLogSpGroup1SmallDecayClass2>");
		oOut.write("<su_initialLogSpGroup1SmallDecayClass3>0.0013</su_initialLogSpGroup1SmallDecayClass3>");
		oOut.write("<su_initialLogSpGroup1SmallDecayClass4>0.0014</su_initialLogSpGroup1SmallDecayClass4>");
		oOut.write("<su_initialLogSpGroup1SmallDecayClass5>0.0015</su_initialLogSpGroup1SmallDecayClass5>");
		oOut.write("<su_initialLogSpGroup1LargeDecayClass1>0.0016</su_initialLogSpGroup1LargeDecayClass1>");
		oOut.write("<su_initialLogSpGroup1LargeDecayClass2>0.0017</su_initialLogSpGroup1LargeDecayClass2>");
		oOut.write("<su_initialLogSpGroup1LargeDecayClass3>0.0018</su_initialLogSpGroup1LargeDecayClass3>");
		oOut.write("<su_initialLogSpGroup1LargeDecayClass4>0.0019</su_initialLogSpGroup1LargeDecayClass4>");
		oOut.write("<su_initialLogSpGroup1LargeDecayClass5>0.002</su_initialLogSpGroup1LargeDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup1SmallDecayClass1>0.0021</su_partialCutLogSpGroup1SmallDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup1SmallDecayClass2>0.0022</su_partialCutLogSpGroup1SmallDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup1SmallDecayClass3>0.0023</su_partialCutLogSpGroup1SmallDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup1SmallDecayClass4>0.0024</su_partialCutLogSpGroup1SmallDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup1SmallDecayClass5>0.0025</su_partialCutLogSpGroup1SmallDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup1LargeDecayClass1>0.0026</su_partialCutLogSpGroup1LargeDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup1LargeDecayClass2>0.0027</su_partialCutLogSpGroup1LargeDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup1LargeDecayClass3>0.0028</su_partialCutLogSpGroup1LargeDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup1LargeDecayClass4>0.0029</su_partialCutLogSpGroup1LargeDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup1LargeDecayClass5>0.0030</su_partialCutLogSpGroup1LargeDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup1SmallDecayClass1>0.0031</su_gapCutLogSpGroup1SmallDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup1SmallDecayClass2>0.0032</su_gapCutLogSpGroup1SmallDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup1SmallDecayClass3>0.0033</su_gapCutLogSpGroup1SmallDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup1SmallDecayClass4>0.0034</su_gapCutLogSpGroup1SmallDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup1SmallDecayClass5>0.0035</su_gapCutLogSpGroup1SmallDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup1LargeDecayClass1>0.0036</su_gapCutLogSpGroup1LargeDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup1LargeDecayClass2>0.0037</su_gapCutLogSpGroup1LargeDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup1LargeDecayClass3>0.0038</su_gapCutLogSpGroup1LargeDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup1LargeDecayClass4>0.0039</su_gapCutLogSpGroup1LargeDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup1LargeDecayClass5>0.0040</su_gapCutLogSpGroup1LargeDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup1SmallDecayClass1>0.0041</su_clearCutLogSpGroup1SmallDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup1SmallDecayClass2>0.0042</su_clearCutLogSpGroup1SmallDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup1SmallDecayClass3>0.0043</su_clearCutLogSpGroup1SmallDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup1SmallDecayClass4>0.0044</su_clearCutLogSpGroup1SmallDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup1SmallDecayClass5>0.0045</su_clearCutLogSpGroup1SmallDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup1LargeDecayClass1>0.0046</su_clearCutLogSpGroup1LargeDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup1LargeDecayClass2>0.0047</su_clearCutLogSpGroup1LargeDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup1LargeDecayClass3>0.0048</su_clearCutLogSpGroup1LargeDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup1LargeDecayClass4>0.0049</su_clearCutLogSpGroup1LargeDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup1LargeDecayClass5>0.0050</su_clearCutLogSpGroup1LargeDecayClass5>");
		oOut.write("<su_logSpGroup2SmallDecayClass1DecayAlpha>-18</su_logSpGroup2SmallDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup2SmallDecayClass2DecayAlpha>-19</su_logSpGroup2SmallDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup2SmallDecayClass3DecayAlpha>-20</su_logSpGroup2SmallDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup2SmallDecayClass4DecayAlpha>-21</su_logSpGroup2SmallDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup2SmallDecayClass5DecayAlpha>-22</su_logSpGroup2SmallDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup2LargeDecayClass1DecayAlpha>-23</su_logSpGroup2LargeDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup2LargeDecayClass2DecayAlpha>-24</su_logSpGroup2LargeDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup2LargeDecayClass3DecayAlpha>-25</su_logSpGroup2LargeDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup2LargeDecayClass4DecayAlpha>-26</su_logSpGroup2LargeDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup2LargeDecayClass5DecayAlpha>-27</su_logSpGroup2LargeDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup2SmallDecayClass1DecayBeta>0.012</su_logSpGroup2SmallDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup2SmallDecayClass2DecayBeta>0.013</su_logSpGroup2SmallDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup2SmallDecayClass3DecayBeta>0.014</su_logSpGroup2SmallDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup2SmallDecayClass4DecayBeta>0.015</su_logSpGroup2SmallDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup2SmallDecayClass5DecayBeta>0.016</su_logSpGroup2SmallDecayClass5DecayBeta>");
		oOut.write("<su_logSpGroup2LargeDecayClass1DecayBeta>0.017</su_logSpGroup2LargeDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup2LargeDecayClass2DecayBeta>0.018</su_logSpGroup2LargeDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup2LargeDecayClass3DecayBeta>0.019</su_logSpGroup2LargeDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup2LargeDecayClass4DecayBeta>0.020</su_logSpGroup2LargeDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup2LargeDecayClass5DecayBeta>0.021</su_logSpGroup2LargeDecayClass5DecayBeta>");
		oOut.write("<su_initialLogSpGroup2SmallDecayClass1>0.0051</su_initialLogSpGroup2SmallDecayClass1>");
		oOut.write("<su_initialLogSpGroup2SmallDecayClass2>0.0052</su_initialLogSpGroup2SmallDecayClass2>");
		oOut.write("<su_initialLogSpGroup2SmallDecayClass3>0.0053</su_initialLogSpGroup2SmallDecayClass3>");
		oOut.write("<su_initialLogSpGroup2SmallDecayClass4>0.0054</su_initialLogSpGroup2SmallDecayClass4>");
		oOut.write("<su_initialLogSpGroup2SmallDecayClass5>0.0055</su_initialLogSpGroup2SmallDecayClass5>");
		oOut.write("<su_initialLogSpGroup2LargeDecayClass1>0.0056</su_initialLogSpGroup2LargeDecayClass1>");
		oOut.write("<su_initialLogSpGroup2LargeDecayClass2>0.0057</su_initialLogSpGroup2LargeDecayClass2>");
		oOut.write("<su_initialLogSpGroup2LargeDecayClass3>0.0058</su_initialLogSpGroup2LargeDecayClass3>");
		oOut.write("<su_initialLogSpGroup2LargeDecayClass4>0.0059</su_initialLogSpGroup2LargeDecayClass4>");
		oOut.write("<su_initialLogSpGroup2LargeDecayClass5>0.0060</su_initialLogSpGroup2LargeDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup2SmallDecayClass1>0.0061</su_partialCutLogSpGroup2SmallDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup2SmallDecayClass2>0.0062</su_partialCutLogSpGroup2SmallDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup2SmallDecayClass3>0.0063</su_partialCutLogSpGroup2SmallDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup2SmallDecayClass4>0.0064</su_partialCutLogSpGroup2SmallDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup2SmallDecayClass5>0.0065</su_partialCutLogSpGroup2SmallDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup2LargeDecayClass1>0.0066</su_partialCutLogSpGroup2LargeDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup2LargeDecayClass2>0.0067</su_partialCutLogSpGroup2LargeDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup2LargeDecayClass3>0.0068</su_partialCutLogSpGroup2LargeDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup2LargeDecayClass4>0.0069</su_partialCutLogSpGroup2LargeDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup2LargeDecayClass5>0.0070</su_partialCutLogSpGroup2LargeDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup2SmallDecayClass1>0.0071</su_gapCutLogSpGroup2SmallDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup2SmallDecayClass2>0.0072</su_gapCutLogSpGroup2SmallDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup2SmallDecayClass3>0.0073</su_gapCutLogSpGroup2SmallDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup2SmallDecayClass4>0.0074</su_gapCutLogSpGroup2SmallDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup2SmallDecayClass5>0.0075</su_gapCutLogSpGroup2SmallDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup2LargeDecayClass1>0.0076</su_gapCutLogSpGroup2LargeDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup2LargeDecayClass2>0.0077</su_gapCutLogSpGroup2LargeDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup2LargeDecayClass3>0.0078</su_gapCutLogSpGroup2LargeDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup2LargeDecayClass4>0.0079</su_gapCutLogSpGroup2LargeDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup2LargeDecayClass5>0.0080</su_gapCutLogSpGroup2LargeDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup2SmallDecayClass1>0.0081</su_clearCutLogSpGroup2SmallDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup2SmallDecayClass2>0.0082</su_clearCutLogSpGroup2SmallDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup2SmallDecayClass3>0.0083</su_clearCutLogSpGroup2SmallDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup2SmallDecayClass4>0.0084</su_clearCutLogSpGroup2SmallDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup2SmallDecayClass5>0.0085</su_clearCutLogSpGroup2SmallDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup2LargeDecayClass1>0.0086</su_clearCutLogSpGroup2LargeDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup2LargeDecayClass2>0.0087</su_clearCutLogSpGroup2LargeDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup2LargeDecayClass3>0.0088</su_clearCutLogSpGroup2LargeDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup2LargeDecayClass4>0.0089</su_clearCutLogSpGroup2LargeDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup2LargeDecayClass5>0.0090</su_clearCutLogSpGroup2LargeDecayClass5>");
		oOut.write("<su_logSpGroup3SmallDecayClass1DecayAlpha>-0.1</su_logSpGroup3SmallDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup3SmallDecayClass2DecayAlpha>-0.2</su_logSpGroup3SmallDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup3SmallDecayClass3DecayAlpha>-0.3</su_logSpGroup3SmallDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup3SmallDecayClass4DecayAlpha>-0.4</su_logSpGroup3SmallDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup3SmallDecayClass5DecayAlpha>-0.5</su_logSpGroup3SmallDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup3LargeDecayClass1DecayAlpha>-0.6</su_logSpGroup3LargeDecayClass1DecayAlpha>");
		oOut.write("<su_logSpGroup3LargeDecayClass2DecayAlpha>-0.7</su_logSpGroup3LargeDecayClass2DecayAlpha>");
		oOut.write("<su_logSpGroup3LargeDecayClass3DecayAlpha>-0.8</su_logSpGroup3LargeDecayClass3DecayAlpha>");
		oOut.write("<su_logSpGroup3LargeDecayClass4DecayAlpha>-0.9</su_logSpGroup3LargeDecayClass4DecayAlpha>");
		oOut.write("<su_logSpGroup3LargeDecayClass5DecayAlpha>-0.11</su_logSpGroup3LargeDecayClass5DecayAlpha>");
		oOut.write("<su_logSpGroup3SmallDecayClass1DecayBeta>-0.12</su_logSpGroup3SmallDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup3SmallDecayClass2DecayBeta>-0.13</su_logSpGroup3SmallDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup3SmallDecayClass3DecayBeta>-0.14</su_logSpGroup3SmallDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup3SmallDecayClass4DecayBeta>-0.15</su_logSpGroup3SmallDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup3SmallDecayClass5DecayBeta>-0.16</su_logSpGroup3SmallDecayClass5DecayBeta>");
		oOut.write("<su_logSpGroup3LargeDecayClass1DecayBeta>-0.17</su_logSpGroup3LargeDecayClass1DecayBeta>");
		oOut.write("<su_logSpGroup3LargeDecayClass2DecayBeta>-0.18</su_logSpGroup3LargeDecayClass2DecayBeta>");
		oOut.write("<su_logSpGroup3LargeDecayClass3DecayBeta>-0.19</su_logSpGroup3LargeDecayClass3DecayBeta>");
		oOut.write("<su_logSpGroup3LargeDecayClass4DecayBeta>-0.21</su_logSpGroup3LargeDecayClass4DecayBeta>");
		oOut.write("<su_logSpGroup3LargeDecayClass5DecayBeta>-0.22</su_logSpGroup3LargeDecayClass5DecayBeta>");
		oOut.write("<su_initialLogSpGroup3SmallDecayClass1>0.0091</su_initialLogSpGroup3SmallDecayClass1>");
		oOut.write("<su_initialLogSpGroup3SmallDecayClass2>0.0092</su_initialLogSpGroup3SmallDecayClass2>");
		oOut.write("<su_initialLogSpGroup3SmallDecayClass3>0.0093</su_initialLogSpGroup3SmallDecayClass3>");
		oOut.write("<su_initialLogSpGroup3SmallDecayClass4>0.0094</su_initialLogSpGroup3SmallDecayClass4>");
		oOut.write("<su_initialLogSpGroup3SmallDecayClass5>0.0095</su_initialLogSpGroup3SmallDecayClass5>");
		oOut.write("<su_initialLogSpGroup3LargeDecayClass1>0.0096</su_initialLogSpGroup3LargeDecayClass1>");
		oOut.write("<su_initialLogSpGroup3LargeDecayClass2>0.0097</su_initialLogSpGroup3LargeDecayClass2>");
		oOut.write("<su_initialLogSpGroup3LargeDecayClass3>0.0098</su_initialLogSpGroup3LargeDecayClass3>");
		oOut.write("<su_initialLogSpGroup3LargeDecayClass4>0.0099</su_initialLogSpGroup3LargeDecayClass4>");
		oOut.write("<su_initialLogSpGroup3LargeDecayClass5>0.0001</su_initialLogSpGroup3LargeDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup3SmallDecayClass1>0.0002</su_partialCutLogSpGroup3SmallDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup3SmallDecayClass2>0.0003</su_partialCutLogSpGroup3SmallDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup3SmallDecayClass3>0.0004</su_partialCutLogSpGroup3SmallDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup3SmallDecayClass4>0.0005</su_partialCutLogSpGroup3SmallDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup3SmallDecayClass5>0.0006</su_partialCutLogSpGroup3SmallDecayClass5>");
		oOut.write("<su_partialCutLogSpGroup3LargeDecayClass1>0.0007</su_partialCutLogSpGroup3LargeDecayClass1>");
		oOut.write("<su_partialCutLogSpGroup3LargeDecayClass2>0.0008</su_partialCutLogSpGroup3LargeDecayClass2>");
		oOut.write("<su_partialCutLogSpGroup3LargeDecayClass3>0.0009</su_partialCutLogSpGroup3LargeDecayClass3>");
		oOut.write("<su_partialCutLogSpGroup3LargeDecayClass4>0.00011</su_partialCutLogSpGroup3LargeDecayClass4>");
		oOut.write("<su_partialCutLogSpGroup3LargeDecayClass5>0.00012</su_partialCutLogSpGroup3LargeDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup3SmallDecayClass1>0.00013</su_gapCutLogSpGroup3SmallDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup3SmallDecayClass2>0.00014</su_gapCutLogSpGroup3SmallDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup3SmallDecayClass3>0.00015</su_gapCutLogSpGroup3SmallDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup3SmallDecayClass4>0.00016</su_gapCutLogSpGroup3SmallDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup3SmallDecayClass5>0.00017</su_gapCutLogSpGroup3SmallDecayClass5>");
		oOut.write("<su_gapCutLogSpGroup3LargeDecayClass1>0.00018</su_gapCutLogSpGroup3LargeDecayClass1>");
		oOut.write("<su_gapCutLogSpGroup3LargeDecayClass2>0.00019</su_gapCutLogSpGroup3LargeDecayClass2>");
		oOut.write("<su_gapCutLogSpGroup3LargeDecayClass3>0.00021</su_gapCutLogSpGroup3LargeDecayClass3>");
		oOut.write("<su_gapCutLogSpGroup3LargeDecayClass4>0.00022</su_gapCutLogSpGroup3LargeDecayClass4>");
		oOut.write("<su_gapCutLogSpGroup3LargeDecayClass5>0.00023</su_gapCutLogSpGroup3LargeDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup3SmallDecayClass1>0.00024</su_clearCutLogSpGroup3SmallDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup3SmallDecayClass2>0.00025</su_clearCutLogSpGroup3SmallDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup3SmallDecayClass3>0.00026</su_clearCutLogSpGroup3SmallDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup3SmallDecayClass4>0.00027</su_clearCutLogSpGroup3SmallDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup3SmallDecayClass5>0.00028</su_clearCutLogSpGroup3SmallDecayClass5>");
		oOut.write("<su_clearCutLogSpGroup3LargeDecayClass1>0.00029</su_clearCutLogSpGroup3LargeDecayClass1>");
		oOut.write("<su_clearCutLogSpGroup3LargeDecayClass2>0.00031</su_clearCutLogSpGroup3LargeDecayClass2>");
		oOut.write("<su_clearCutLogSpGroup3LargeDecayClass3>0.00032</su_clearCutLogSpGroup3LargeDecayClass3>");
		oOut.write("<su_clearCutLogSpGroup3LargeDecayClass4>0.00033</su_clearCutLogSpGroup3LargeDecayClass4>");
		oOut.write("<su_clearCutLogSpGroup3LargeDecayClass5>0.00034</su_clearCutLogSpGroup3LargeDecayClass5>");
		oOut.write("<su_gapCutScarSoil>0.035</su_gapCutScarSoil>");
		oOut.write("<su_gapCutTipup>0.036</su_gapCutTipup>");
		oOut.write("<su_partialCutScarSoil>0.037</su_partialCutScarSoil>");
		oOut.write("<su_partialCutTipup>0.038</su_partialCutTipup>");
		oOut.write("<su_initialScarSoil>0.039</su_initialScarSoil>");
		oOut.write("<su_initialTipup>0.041</su_initialTipup>");
		oOut.write("<su_clearCutScarSoil>0.042</su_clearCutScarSoil>");
		oOut.write("<su_clearCutTipup>0.043</su_clearCutTipup>");
		oOut.write("<su_scarSoilDecayAlpha>-0.044</su_scarSoilDecayAlpha>");
		oOut.write("<su_scarSoilDecayBeta>0.045</su_scarSoilDecayBeta>");
		oOut.write("<su_tipupDecayAlpha>-0.046</su_tipupDecayAlpha>");
		oOut.write("<su_tipupDecayBeta>0.047</su_tipupDecayBeta>");
		oOut.write("<su_maxNumberDecayYears>2</su_maxNumberDecayYears>");
		oOut.write("<su_propOfFallUproot>");
		oOut.write("<su_pofuVal species=\"Species_1\">0.048</su_pofuVal>");
		oOut.write("<su_pofuVal species=\"Species_2\">0.049</su_pofuVal>");
		oOut.write("<su_pofuVal species=\"Species_3\">0.053</su_pofuVal>");
		oOut.write("</su_propOfFallUproot>");
		oOut.write("<su_propOfSnagsUproot>");
		oOut.write("<su_posuVal species=\"Species_1\">0.051</su_posuVal>");
		oOut.write("<su_posuVal species=\"Species_2\">0.052</su_posuVal>");
		oOut.write("<su_posuVal species=\"Species_3\">0.054</su_posuVal>");
		oOut.write("</su_propOfSnagsUproot>");
		oOut.write("<su_rootTipupFactor>3</su_rootTipupFactor>");
		oOut.write("<su_mossProportion>0.4</su_mossProportion>");
		oOut.write("<su_directionalTreeFall>0</su_directionalTreeFall>");
		oOut.write("<su_initialSmallLogMeanDiameter>20</su_initialSmallLogMeanDiameter>");
		oOut.write("<su_initialLargeLogMeanDiameter>50</su_initialLargeLogMeanDiameter>");
		oOut.write("<su_partialCutSmallLogMeanDiameter>21</su_partialCutSmallLogMeanDiameter>");
		oOut.write("<su_partialCutLargeLogMeanDiameter>51</su_partialCutLargeLogMeanDiameter>");
		oOut.write("<su_gapCutSmallLogMeanDiameter>22</su_gapCutSmallLogMeanDiameter>");
		oOut.write("<su_gapCutLargeLogMeanDiameter>52</su_gapCutLargeLogMeanDiameter>");
		oOut.write("<su_clearCutSmallLogMeanDiameter>23</su_clearCutSmallLogMeanDiameter>");
		oOut.write("<su_clearCutLargeLogMeanDiameter>53</su_clearCutLargeLogMeanDiameter>");
		oOut.write("</substrate>");

		oOut.write("</paramFile>");
		oOut.close();
		return sFileName;
	}
}
